@@ -11,6 +11,7 @@ import (
 	"fmt"
 	"net/http"
 	"net/http/httptest"
+	"runtime"
 	"testing"
 
 	"ariga.io/atlas/sql/migrate"
@@ -34,7 +35,8 @@ func TestClient_Dir(t *testing.T) {
 		require.Equal(t, "foo", input.Variables.DirInput.Name)
 		require.Equal(t, "x", input.Variables.DirInput.Tag)
 		require.Equal(t, "Bearer atlas", r.Header.Get("Authorization"))
-		require.Equal(t, "Atlas/v0.13.0", r.Header.Get("User-Agent"))
+		expUA := fmt.Sprintf("Atlas/v0.13.0 (%s/%s)", runtime.GOOS, runtime.GOARCH)
+		require.Equal(t, expUA, r.Header.Get("User-Agent"))
 		fmt.Fprintf(w, `{"data":{"dirState":{"content":%q}}}`, base64.StdEncoding.EncodeToString(ad))
 	}))
 	client := New(srv.URL, "atlas")
@@ -137,3 +139,12 @@ func TestClient_ReportMigrationSet(t *testing.T) {
 	require.NoError(t, err)
 	require.NotEmpty(t, link)
 }
+
+func TestRedactedURL(t *testing.T) {
+	u, err := RedactedURL("mysql://user:pass@:3306/db")
+	require.NoError(t, err)
+	require.Equal(t, "mysql://user:xxxxx@:3306/db", u)
+	u, err = RedactedURL("\\n mysql://user:pass@:3306/db")
+	require.EqualError(t, err, `first path segment in URL cannot contain colon`)
+	require.Empty(t, u)
+}
