@@ -26,10 +26,8 @@ import (
 	"ariga.io/atlas/cmd/atlas/internal/cmdlog"
 	cmdmigrate "ariga.io/atlas/cmd/atlas/internal/migrate"
 	"ariga.io/atlas/cmd/atlas/internal/migrate/ent/revision"
-	"ariga.io/atlas/cmd/atlas/internal/migratelint"
 	"ariga.io/atlas/sql/migrate"
 	"ariga.io/atlas/sql/schema"
-	"ariga.io/atlas/sql/sqlcheck"
 	"ariga.io/atlas/sql/sqlclient"
 	"ariga.io/atlas/sql/sqltool"
 
@@ -87,8 +85,8 @@ As a safety measure 'atlas migrate apply' will abort with an error, if:
   - the migration and database history do not match each other
 
 If run with the "--dry-run" flag, atlas will not execute any SQL.`,
-			Example: `  atlas migrate apply -u mysql://user:pass@localhost:3306/dbname
-  atlas migrate apply --dir file:///path/to/migration/directory --url mysql://user:pass@localhost:3306/dbname 1
+			Example: `  atlas migrate apply -u "mysql://user:pass@localhost:3306/dbname"
+  atlas migrate apply --dir "file:///path/to/migration/directory" --url "mysql://user:pass@localhost:3306/dbname" 1
   atlas migrate apply --env dev 1
   atlas migrate apply --dry-run --env dev 1`,
 			Args: cobra.MaximumNArgs(1),
@@ -100,13 +98,24 @@ If run with the "--dry-run" flag, atlas will not execute any SQL.`,
 					}
 					return migrateApplyRun(cmd, args, flags, &MigrateReport{}) // nop reporter
 				default:
-					project, envs, err := EnvByName(GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
+					project, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
 					if err != nil {
 						return err
 					}
-					set := NewReportProvider(cmd.Context(), project, envs)
-					defer func() { set.Flush(cmd, cmdErr) }()
+					var (
+						hasRemote bool
+						set       = NewReportProvider(cmd.Context(), project, envs)
+					)
+					defer func() {
+						if hasRemote {
+							set.Flush(cmd, cmdErr)
+						}
+					}()
 					return cmdEnvsRun(envs, setMigrateEnvFlags, cmd, func(env *Env) error {
+						// Report deployments only if one of the migration directories is a cloud directory.
+						if u, err := url.Parse(flags.dirURL); err == nil && u.Scheme != cmdmigrate.DirTypeFile {
+							hasRemote = true
+						}
 						return migrateApplyRun(cmd, args, flags, set.ReportFor(flags, env))
 					})
 				}
@@ -186,7 +195,7 @@ func migrateApplyRun(cmd *cobra.Command, args []string, flags migrateApplyFlags,
 		return err
 	}
 	// Setup reporting info.
-	report := cmdlog.NewMigrateApply(client, dir)
+	report := cmdlog.NewMigrateApply(ctx, client, dir)
 	mr.Init(client, report, mrrw)
 	// If cloud reporting is enabled, and we cannot obtain the current
 	// target identifier, abort and report it to the user.
@@ -359,6 +368,11 @@ func (s *MigrateReportSet) ReportFor(flags migrateApplyFlags, e *Env) *MigrateRe
 // Note that reporting errors are logged, but not cause Atlas to fail.
 func (s *MigrateReportSet) Flush(cmd *cobra.Command, cmdErr error) {
 	if cmdErr != nil && s.Error == nil {
+		var uerr *url.Error
+		if errors.As(cmdErr, &uerr) {
+			uerr.URL = ""
+			cmdErr = uerr
+		}
 		s.StepLogError(cmdErr.Error())
 	}
 	var (
@@ -433,7 +447,14 @@ func (r *MigrateReport) Done(cmd *cobra.Command, flags migrateApplyFlags) error
 		ver = rev.Version
 	}
 	dirName := flags.dirURL
-	if u, err := url.Parse(flags.dirURL); err == nil {
+	switch u, err := url.Parse(flags.dirURL); {
+	case err != nil:
+	// Local directories are reported as (dangling)
+	// deployments without a directory.
+	case u.Scheme == cmdmigrate.DirTypeFile:
+		dirName = cloudapi.DefaultDirName
+	// Directory slug.
+	default:
 		dirName = filepath.Join(u.Host, u.Path)
 	}
 	r.done(&cloudapi.ReportMigrationInput{
@@ -524,9 +545,9 @@ func migrateDiffCmd() *cobra.Command {
 by executing its files. It then compares its state to the desired state and create a new migration file containing
 SQL statements for moving from the current to the desired state. The desired state can be another another database,
 an HCL, SQL, or ORM schema. See: https://atlasgo.io/versioned/diff`,
-			Example: `  atlas migrate diff --dev-url docker://mysql/8/dev --to file://schema.hcl
-  atlas migrate diff --dev-url "docker://postgres/15/dev?search_path=public" --to file://atlas.hcl add_users_table
-  atlas migrate diff --dev-url mysql://user:pass@localhost:3306/dev --to mysql://user:pass@localhost:3306/dbname
+			Example: `  atlas migrate diff "--dev-url docker://mysql/8/dev" --to "file://schema.hcl"
+  atlas migrate diff --dev-url "docker://postgres/15/dev?search_path=public" --to "file://atlas.hcl" add_users_table
+  atlas migrate diff --dev-url "mysql://user:pass@localhost:3306/dev" --to "mysql://user:pass@localhost:3306/dbname"
   atlas migrate diff --env dev --format '{{ sql . "  " }}'`,
 			Args: cobra.MaximumNArgs(1),
 			PreRunE: func(cmd *cobra.Command, args []string) error {
@@ -759,7 +780,7 @@ func migrateImportCmd() *cobra.Command {
 		cmd   = &cobra.Command{
 			Use:     "import [flags]",
 			Short:   "Import a migration directory from another migration management tool to the Atlas format.",
-			Example: `  atlas migrate import --from file:///path/to/source/directory?format=liquibase --to file:///path/to/migration/directory`,
+			Example: `  atlas migrate import --from "file:///path/to/source/directory?format=liquibase" --to "file:///path/to/migration/directory"`,
 			// Validate the source directory. Consider a directory with no sum file
 			// valid, since it might be an import from an existing project.
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
@@ -873,8 +894,12 @@ type migrateLintFlags struct {
 	dirURL, dirFormat string
 	devURL            string
 	logFormat         string
-	latest            uint
-	gitBase, gitDir   string
+	latest            uint   // --latest 1
+	gitBase, gitDir   string // --git-base master --git-dir /path/to/git/repo
+	// Not enabled by default.
+	dirBase string // --base atlas://graph
+	web     bool   // Open the web browser
+	context string // Run context. See cloudapi.ContextInput.
 }
 
 // migrateLintCmd represents the 'atlas migrate lint' subcommand.
@@ -885,9 +910,9 @@ func migrateLintCmd() *cobra.Command {
 			Use:   "lint [flags]",
 			Short: "Run analysis on the migration directory",
 			Example: `  atlas migrate lint --env dev
-  atlas migrate lint --dir file:///path/to/migration/directory --dev-url mysql://root:pass@localhost:3306 --latest 1
-  atlas migrate lint --dir file:///path/to/migration/directory --dev-url mysql://root:pass@localhost:3306 --git-base master
-  atlas migrate lint --dir file:///path/to/migration/directory --dev-url mysql://root:pass@localhost:3306 --format '{{ json .Files }}'`,
+  atlas migrate lint --dir "file:///path/to/migrations" --dev-url "docker://mysql/8/dev" --latest 1
+  atlas migrate lint --dir "file:///path/to/migrations" --dev-url "mysql://root:pass@localhost:3306" --git-base master
+  atlas migrate lint --dir "file:///path/to/migrations" --dev-url "mysql://root:pass@localhost:3306" --format '{{ json .Files }}'`,
 			PreRunE: func(cmd *cobra.Command, args []string) error {
 				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
@@ -910,70 +935,10 @@ func migrateLintCmd() *cobra.Command {
 	cmd.Flags().StringVarP(&flags.gitDir, flagGitDir, "", ".", "path to the repository working directory")
 	cobra.CheckErr(cmd.MarkFlagRequired(flagDevURL))
 	cmd.MarkFlagsMutuallyExclusive(flagLog, flagFormat)
+	migrateLintSetFlags(cmd, &flags)
 	return cmd
 }
 
-func migrateLintRun(cmd *cobra.Command, _ []string, flags migrateLintFlags) error {
-	dev, err := sqlclient.Open(cmd.Context(), flags.devURL)
-	if err != nil {
-		return err
-	}
-	defer dev.Close()
-	dir, err := cmdmigrate.Dir(cmd.Context(), flags.dirURL, false)
-	if err != nil {
-		return err
-	}
-	var detect migratelint.ChangeDetector
-	switch {
-	case flags.latest == 0 && flags.gitBase == "":
-		return fmt.Errorf("--%s or --%s is required", flagLatest, flagGitBase)
-	case flags.latest > 0 && flags.gitBase != "":
-		return fmt.Errorf("--%s and --%s are mutually exclusive", flagLatest, flagGitBase)
-	case flags.latest > 0:
-		detect = migratelint.LatestChanges(dir, int(flags.latest))
-	case flags.gitBase != "":
-		detect, err = migratelint.NewGitChangeDetector(
-			dir,
-			migratelint.WithWorkDir(flags.gitDir),
-			migratelint.WithBase(flags.gitBase),
-			migratelint.WithMigrationsPath(dir.(interface{ Path() string }).Path()),
-		)
-		if err != nil {
-			return err
-		}
-	}
-	format := migratelint.DefaultTemplate
-	if f := flags.logFormat; f != "" {
-		format, err = template.New("format").Funcs(migratelint.TemplateFuncs).Parse(f)
-		if err != nil {
-			return fmt.Errorf("parse format: %w", err)
-		}
-	}
-	env, err := selectEnv(cmd)
-	if err != nil {
-		return err
-	}
-	az, err := sqlcheck.AnalyzerFor(dev.Name, env.Lint.Remain())
-	if err != nil {
-		return err
-	}
-	r := &migratelint.Runner{
-		Dev:            dev,
-		Dir:            dir,
-		ChangeDetector: detect,
-		ReportWriter: &migratelint.TemplateWriter{
-			T: format,
-			W: cmd.OutOrStdout(),
-		},
-		Analyzers: az,
-	}
-	err = r.Run(cmd.Context())
-	// Print the error in case it was not printed before.
-	cmd.SilenceErrors = errors.As(err, &migratelint.SilentError{})
-	cmd.SilenceUsage = cmd.SilenceErrors
-	return err
-}
-
 type migrateNewFlags struct {
 	edit      bool
 	dirURL    string
@@ -1049,9 +1014,9 @@ func migrateSetCmd() *cobra.Command {
 			Short: "Set the current version of the migration history table.",
 			Long: `'atlas migrate set' edits the revision table to consider all migrations up to and including the given version
 to be applied. This command is usually used after manually making changes to the managed database.`,
-			Example: `  atlas migrate set 3 --url mysql://user:pass@localhost:3306/
+			Example: `  atlas migrate set 3 --url "mysql://user:pass@localhost:3306/"
   atlas migrate set --env local
-  atlas migrate set 1.2.4 --url mysql://user:pass@localhost:3306/my_db --revision-schema my_revisions`,
+  atlas migrate set 1.2.4 --url "mysql://user:pass@localhost:3306/my_db" --revision-schema my_revisions`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
@@ -1150,7 +1115,7 @@ func migrateSetRun(cmd *cobra.Command, args []string, flags migrateSetFlags) (re
 	default:
 		return fmt.Errorf("accepts 1 arg(s), received %d", n)
 	}
-	log := &cmdlog.MigrateSet{}
+	log := &cmdlog.MigrateSet{Context: ctx}
 	for _, r := range revs {
 		// Check all existing revisions and ensure they precede the given version. If we encounter a partially
 		// applied revision, or one with errors, mark them "fixed".
@@ -1245,8 +1210,8 @@ func migrateStatusCmd() *cobra.Command {
 			Use:   "status [flags]",
 			Short: "Get information about the current migration status.",
 			Long:  `'atlas migrate status' reports information about the current status of a connected database compared to the migration directory.`,
-			Example: `  atlas migrate status --url mysql://user:pass@localhost:3306/
-  atlas migrate status --url mysql://user:pass@localhost:3306/ --dir file:///path/to/migration/directory`,
+			Example: `  atlas migrate status --url "mysql://user:pass@localhost:3306/"
+  atlas migrate status --url "mysql://user:pass@localhost:3306/" --dir "file:///path/to/migration/directory"`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
@@ -1319,9 +1284,9 @@ func migrateValidateCmd() *cobra.Command {
 atlas.sum file. If there is a mismatch it will be reported. If the --dev-url flag is given, the migration
 files are executed on the connected database in order to validate SQL semantics.`,
 			Example: `  atlas migrate validate
-  atlas migrate validate --dir file:///path/to/migration/directory
-  atlas migrate validate --dir file:///path/to/migration/directory --dev-url mysql://user:pass@localhost:3306/dev
-  atlas migrate validate --env dev`,
+  atlas migrate validate --dir "file:///path/to/migration/directory"
+  atlas migrate validate --dir "file:///path/to/migration/directory" --dev-url "docker://mysql/8/dev"
+  atlas migrate validate --env dev --dev-url "docker://postgres/15/dev?search_path=public"`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				if err := migrateFlagsFromConfig(cmd); err != nil {
 					return err
