@@ -802,6 +802,8 @@ func TestMigrate_ApplyCloudReport(t *testing.T) {
 			}
 			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
 			switch {
+			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "bot"):
+				fmt.Fprintln(w, `{"data": {"me":{ "name": "a8m", "org": "graph"}}}`)
 			case strings.Contains(m.Query, "query"):
 				// Checksum before archiving.
 				hf, err := dir.Checksum()
@@ -970,6 +972,37 @@ env {
 		require.EqualError(t, err, "openerror")
 		require.Equal(t, "Error: openerror", *reports.Error)
 	})
+
+	t.Run("RedactedURL", func(t *testing.T) {
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		_, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--url", "    sqlite://my:pass@file.db",
+			"--var", "cloud_url="+srv.URL,
+		)
+		// Open errors are reported without URLs.
+		require.EqualError(t, err, `sql/sqlclient: parse open url: first path segment in URL cannot contain colon`)
+		require.Equal(t, `Error: sql/sqlclient: parse open url: first path segment in URL cannot contain colon`, *reports.Error)
+	})
+
+	t.Run("LocalDirectory", func(t *testing.T) {
+		report.DirName = "NotReported"
+		cmd := migrateCmd()
+		cmd.AddCommand(migrateApplyCmd())
+		_, err := runCmd(
+			cmd, "apply",
+			"-c", "file://"+path,
+			"--env", "local",
+			"--dir", "file://"+t.TempDir(),
+			"--url", openSQLite(t, ""),
+			"--var", "cloud_url="+srv.URL,
+		)
+		require.NoError(t, err)
+		require.Equal(t, "NotReported", report.DirName)
+	})
 }
 
 func TestMigrate_ApplyCloudReportSet(t *testing.T) {
@@ -986,6 +1019,8 @@ func TestMigrate_ApplyCloudReportSet(t *testing.T) {
 			}
 			require.NoError(t, json.NewDecoder(r.Body).Decode(&m))
 			switch {
+			case strings.Contains(m.Query, "query") && strings.Contains(m.Query, "bot"):
+				fmt.Fprintln(w, `{"data": {"me":{ "name": "a8m", "org": "graph"}}}`)
 			case strings.Contains(m.Query, "query"):
 				// Checksum before archiving.
 				hf, err := dir.Checksum()
