@@ -9,6 +9,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"io"
 	"io/fs"
 	"os"
 	"path/filepath"
@@ -64,20 +65,18 @@ migration, Atlas will print the migration plan and prompt the user for approval.
 
 The schema is provided by one or more URLs (to a HCL file or 
 directory, database or migration directory) using the "--to, -t" flag:
-  atlas schema apply -u URL --to file://file1.hcl --to file://file2.hcl
-  atlas schema apply -u URL --to file://schema/ --to file://override.hcl
+  atlas schema apply -u URL --to "file://file1.hcl" --to "file://file2.hcl"
+  atlas schema apply -u URL --to "file://schema/" --to "file://override.hcl"
 
 As a convenience, schema URLs may also be provided via an environment definition in
 the project file (see: https://atlasgo.io/cli/projects).
 
 If run with the "--dry-run" flag, atlas will exit after printing out the planned
 migration.`,
-			Example: `  atlas schema apply -u "mysql://user:pass@localhost/dbname" --to file://atlas.hcl
-  atlas schema apply -u "mysql://localhost" --to file://schema.hcl --schema prod --schema staging
-  atlas schema apply -u "mysql://user:pass@localhost:3306/dbname" --to file://schema.hcl --dry-run
-  atlas schema apply -u "mariadb://user:pass@localhost:3306/dbname" --to file://schema.hcl
-  atlas schema apply --url "postgres://user:pass@host:port/dbname?sslmode=disable" --to file://schema.hcl
-  atlas schema apply -u "sqlite://file:ex1.db?_fk=1" --to file://schema.hcl`,
+			Example: `  atlas schema apply -u "mysql://user:pass@localhost/dbname" --to "file://atlas.hcl"
+  atlas schema apply -u "mysql://localhost" --to "file://schema.sql" --dev-url "docker://mysql/8/dev"
+  atlas schema apply --env local --dev-url "docker://postgres/15/dev?search_path=public" --dry-run
+  atlas schema apply -u "sqlite://file.db" --to "file://schema.sql" --dev-url "sqlite://dev?mode=memory"`,
 			RunE: func(cmd *cobra.Command, args []string) error {
 				switch {
 				case GlobalFlags.SelectedEnv == "":
@@ -87,7 +86,7 @@ migration.`,
 					}
 					return schemaApplyRun(cmd, flags, env)
 				default:
-					_, envs, err := EnvByName(GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
+					_, envs, err := EnvByName(cmd, GlobalFlags.SelectedEnv, WithInput(GlobalFlags.Vars))
 					if err != nil {
 						return err
 					}
@@ -166,22 +165,6 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 	if !ok {
 		return errors.New("--url must be a database connection")
 	}
-	apply := client.ApplyChanges
-	if !flags.dryRun && flags.txMode == txModeFile {
-		apply = func(ctx context.Context, cs []schema.Change, _ ...migrate.PlanOption) error {
-			tx, err := client.Tx(ctx, nil)
-			if err != nil {
-				return err
-			}
-			if err := tx.ApplyChanges(ctx, cs); err != nil {
-				// Rollback on error but the underlying error
-				// is still returned to make type-assertion pass.
-				_ = tx.Rollback()
-				return err
-			}
-			return tx.Commit()
-		}
-	}
 	to, err := stateReader(ctx, &stateReaderConfig{
 		urls:    flags.toURLs,
 		dev:     dev,
@@ -198,11 +181,10 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 	if err != nil {
 		return err
 	}
-	changes := diff.changes
 	// Returning at this stage should
 	// not trigger the help message.
 	cmd.SilenceUsage = true
-	switch {
+	switch changes := diff.changes; {
 	case len(changes) == 0:
 		return format.Execute(cmd.OutOrStdout(), &cmdlog.SchemaApply{})
 	case flags.logFormat != "" && flags.autoApprove:
@@ -212,29 +194,67 @@ func schemaApplyRun(cmd *cobra.Command, flags schemaApplyFlags, env *Env) error
 			cause   *cmdlog.StmtError
 			out     = cmd.OutOrStdout()
 		)
-		if plan, err = client.PlanChanges(ctx, "", changes); err != nil {
+		if plan, err = client.PlanChanges(ctx, "", changes, planOptions(client)...); err != nil {
 			return err
 		}
-		if err = apply(ctx, changes); err == nil {
+		if err = applyChanges(ctx, client, changes, flags.txMode); err == nil {
 			applied = len(plan.Changes)
 		} else if i, ok := err.(interface{ Applied() int }); ok && i.Applied() < len(plan.Changes) {
 			applied, cause = i.Applied(), &cmdlog.StmtError{Stmt: plan.Changes[i.Applied()].Cmd, Text: err.Error()}
 		} else {
 			cause = &cmdlog.StmtError{Text: err.Error()}
 		}
-		err1 := format.Execute(out, cmdlog.NewSchemaApply(cmdlog.NewEnv(client, nil), plan.Changes[:applied], plan.Changes[applied:], cause))
+		err1 := format.Execute(out, cmdlog.NewSchemaApply(ctx, cmdlog.NewEnv(client, nil), plan.Changes[:applied], plan.Changes[applied:], cause))
 		return errors.Join(err, err1)
 	default:
-		if err := summary(cmd, client, changes, format); err != nil {
+		switch err := summary(cmd, client, changes, format); {
+		case err != nil:
 			return err
+		case flags.dryRun:
+			return nil
+		case flags.autoApprove:
+			return applyChanges(ctx, client, changes, flags.txMode)
+		default:
+			return promptApply(cmd, flags, diff, client, dev)
 		}
-		if !flags.dryRun && (flags.autoApprove || promptUser()) {
-			return apply(ctx, changes)
-		}
-		return nil
 	}
 }
 
+func applyChanges(ctx context.Context, client *sqlclient.Client, changes []schema.Change, txMode string) error {
+	opts := planOptions(client)
+	if txMode == txModeNone {
+		return client.ApplyChanges(ctx, changes, opts...)
+	}
+	tx, err := client.Tx(ctx, nil)
+	if err != nil {
+		return err
+	}
+	if err := tx.ApplyChanges(ctx, changes, opts...); err != nil {
+		// Rollback on error but the underlying error is still
+		// returned to make type-assertion in schemaApplyRun pass.
+		_ = tx.Rollback()
+		return err
+	}
+	return tx.Commit()
+}
+
+// planOptions returns the default options for planning declarative changes.
+func planOptions(c *sqlclient.Client) []migrate.PlanOption {
+	opts := []migrate.PlanOption{
+		func(opts *migrate.PlanOptions) {
+			opts.Indent = "  "
+		},
+	}
+	// In case the scope is a database schema, generate
+	// the plan without the schema qualifier.
+	if c.URL.Schema != "" {
+		opts = append(opts, func(opt *migrate.PlanOptions) {
+			opt.SchemaQualifier = new(string)
+		})
+	}
+	return opts
+}
+
 type schemeCleanFlags struct {
 	URL         string // URL of database to apply the changes on.
 	AutoApprove bool   // Don't prompt for approval before applying SQL.
@@ -249,8 +269,8 @@ func schemaCleanCmd() *cobra.Command {
 			Short: "Removes all objects from the connected database.",
 			Long: `'atlas schema clean' drops all objects in the connected database and leaves it in an empty state.
 As a safety feature, 'atlas schema clean' will ask for confirmation before attempting to execute any SQL.`,
-			Example: `  atlas schema clean -u mysql://user:pass@localhost:3306/dbname
-  atlas schema clean -u mysql://user:pass@localhost:3306/`,
+			Example: `  atlas schema clean -u "mysql://user:pass@localhost:3306/dbname"
+  atlas schema clean -u "mysql://user:pass@localhost:3306/"`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				return schemaFlagsFromConfig(cmd)
 			},
@@ -302,7 +322,7 @@ func schemaCleanRun(cmd *cobra.Command, _ []string, flags schemeCleanFlags) erro
 	if err := summary(cmd, c, drop, cmdlog.SchemaPlanTemplate); err != nil {
 		return err
 	}
-	if flags.AutoApprove || promptUser() {
+	if flags.AutoApprove || promptUser(cmd) {
 		if err := c.ApplyChanges(cmd.Context(), drop); err != nil {
 			return err
 		}
@@ -335,9 +355,9 @@ func schemaDiffCmdWithFlags() (*cobra.Command, *schemaDiffFlags) {
 calculates the difference in their schemas, and prints a plan of
 SQL statements to migrate the "from" database to the schema of the "to" database.
 The database states can be read from a connected database, an HCL project or a migration directory.`,
-			Example: `  atlas schema diff --from mysql://user:pass@localhost:3306/test --to file://schema.hcl
-  atlas schema diff --from mysql://user:pass@localhost:3306 --to file://schema_1.hcl --to file://schema_2.hcl
-  atlas schema diff --from mysql://user:pass@localhost:3306 --to file://migrations --format '{{ sql . "  " }}'`,
+			Example: `  atlas schema diff --from "mysql://user:pass@localhost:3306/test" --to "file://schema.hcl"
+  atlas schema diff --from "mysql://user:pass@localhost:3306" --to "file://schema_1.hcl" --to "file://schema_2.hcl"
+  atlas schema diff --from "mysql://user:pass@localhost:3306" --to "file://migrations" --format '{{ sql . "  " }}'`,
 			PreRunE: func(cmd *cobra.Command, _ []string) error {
 				return schemaFlagsFromConfig(cmd)
 			},
@@ -415,6 +435,7 @@ func schemaDiffRun(cmd *cobra.Command, _ []string, flags schemaDiffFlags, env *E
 	}
 	return format.Execute(cmd.OutOrStdout(), &cmdlog.SchemaDiff{
 		Client:  c,
+		Context: ctx,
 		From:    diff.from,
 		To:      diff.to,
 		Changes: diff.changes,
@@ -519,8 +540,9 @@ func schemaInspectRun(cmd *cobra.Command, _ []string, flags schemaInspectFlags)
 		return err
 	}
 	return format.Execute(cmd.OutOrStdout(), &cmdlog.SchemaInspect{
-		Client: client,
-		Realm:  s,
+		Client:  client,
+		Context: ctx,
+		Realm:   s,
 	})
 }
 
@@ -572,7 +594,7 @@ func selectEnv(cmd *cobra.Command) (*Env, error) {
 	switch name := GlobalFlags.SelectedEnv; {
 	// A config file was passed without an env.
 	case name == "" && cmd.Flags().Changed(flagConfig):
-		p, envs, err := EnvByName(name, WithInput(GlobalFlags.Vars))
+		p, envs, err := EnvByName(cmd, name, WithInput(GlobalFlags.Vars))
 		if err != nil {
 			return nil, err
 		}
@@ -585,7 +607,7 @@ func selectEnv(cmd *cobra.Command) (*Env, error) {
 		return &Env{Lint: &Lint{}, Migration: &Migration{}}, nil
 	// Env was passed.
 	default:
-		_, envs, err := EnvByName(name, WithInput(GlobalFlags.Vars))
+		_, envs, err := EnvByName(cmd, name, WithInput(GlobalFlags.Vars))
 		if err != nil {
 			return nil, err
 		}
@@ -685,13 +707,13 @@ func computeDiff(ctx context.Context, differ *sqlclient.Client, from, to *cmdext
 }
 
 func summary(cmd *cobra.Command, c *sqlclient.Client, changes []schema.Change, t *template.Template) error {
-	p, err := c.PlanChanges(cmd.Context(), "", changes)
+	p, err := c.PlanChanges(cmd.Context(), "", changes, planOptions(c)...)
 	if err != nil {
 		return err
 	}
 	return t.Execute(
 		cmd.OutOrStdout(),
-		cmdlog.NewSchemaPlan(cmdlog.NewEnv(c, nil), p.Changes, nil),
+		cmdlog.NewSchemaPlan(cmd.Context(), cmdlog.NewEnv(c, nil), p.Changes, nil),
 	)
 }
 
@@ -700,19 +722,30 @@ const (
 	answerAbort = "Abort"
 )
 
-func promptUser() bool {
-	prompt := promptui.Select{
-		Label: "Are you sure?",
-		Items: []string{answerApply, answerAbort},
+// cmdPrompt returns a promptui.Select that uses the given command's input and output.
+func cmdPrompt(cmd *cobra.Command) *promptui.Select {
+	return &promptui.Select{
+		Label:  "Are you sure?",
+		Stdin:  io.NopCloser(cmd.InOrStdin()),
+		Stdout: nopCloser{cmd.OutOrStdout()},
 	}
+}
+
+func promptUser(cmd *cobra.Command) bool {
+	prompt := cmdPrompt(cmd)
+	prompt.Items = []string{answerApply, answerAbort}
 	_, result, err := prompt.Run()
-	if err != nil && err != promptui.ErrInterrupt {
+	if err != nil && !errors.Is(err, promptui.ErrInterrupt) {
 		// Fail in case of unexpected errors.
 		cobra.CheckErr(err)
 	}
 	return result == answerApply
 }
 
+type nopCloser struct{ io.Writer }
+
+func (nopCloser) Close() error { return nil }
+
 func tasks(path string) ([]fmttask, error) {
 	var tasks []fmttask
 	stat, err := os.Stat(path)
