@@ -31,6 +31,8 @@ import (
 	"ariga.io/atlas/sql/sqltool"
 	"github.com/aws/aws-sdk-go-v2/config"
 	"github.com/aws/aws-sdk-go-v2/feature/rds/auth"
+	"golang.org/x/oauth2/google"
+	sqladmin "google.golang.org/api/sqladmin/v1beta4"
 
 	"github.com/zclconf/go-cty/cty/gocty"
 
@@ -42,9 +44,11 @@ import (
 	"github.com/hashicorp/hcl/v2/gohcl"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
 	"github.com/zclconf/go-cty/cty"
+	"gocloud.dev/gcerrors"
 	"gocloud.dev/runtimevar"
 	_ "gocloud.dev/runtimevar/awssecretsmanager"
 	_ "gocloud.dev/runtimevar/constantvar"
+	"gocloud.dev/runtimevar/driver"
 	_ "gocloud.dev/runtimevar/filevar"
 	_ "gocloud.dev/runtimevar/gcpsecretmanager"
 	_ "gocloud.dev/runtimevar/httpvar"
@@ -59,6 +63,7 @@ var DataSources = []schemahcl.Option{
 	schemahcl.WithDataSource("hcl_schema", SchemaHCL),
 	schemahcl.WithDataSource("external_schema", SchemaExternal),
 	schemahcl.WithDataSource("aws_rds_token", AWSRDSToken),
+	schemahcl.WithDataSource("gcp_cloudsql_token", GCPCloudSQLToken),
 }
 
 // RuntimeVar exposes the gocloud.dev/runtimevar as a schemahcl datasource.
@@ -81,7 +86,7 @@ func RuntimeVar(c *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error) {
 	if diags := gohcl.DecodeBody(block.Body, c, &args); diags.HasErrors() {
 		return cty.NilVal, errorf("decoding body: %v", diags)
 	}
-	u, err := url.Parse(args.URL)
+	u, err := sqlclient.ParseURL(args.URL)
 	if err != nil {
 		return cty.NilVal, errorf("parsing url: %v", err)
 	}
@@ -151,6 +156,23 @@ func AWSRDSToken(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error
 	return cty.StringVal(token), nil
 }
 
+// GCPCloudSQLToken exposes a CloudSQL token as a schemahcl datasource.
+//
+//	data "gcp_cloudsql_token" "hello" {}
+func GCPCloudSQLToken(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error) {
+	errorf := blockError("data.gcp_cloudsql_token", block)
+	bgctx := context.Background()
+	ts, err := google.DefaultTokenSource(bgctx, sqladmin.SqlserviceAdminScope)
+	if err != nil {
+		return cty.NilVal, errorf("finding default credentials: %v", err)
+	}
+	token, err := ts.Token()
+	if err != nil {
+		return cty.NilVal, errorf("getting token: %v", err)
+	}
+	return cty.StringVal(token.AccessToken), nil
+}
+
 // Query exposes the database/sql.Query as a schemahcl datasource.
 //
 //	data "sql" "tenants" {
@@ -326,6 +348,9 @@ func TemplateDir(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error
 	if len(attrs) > 0 {
 		return cty.NilVal, errorf("unexpected attributes: %v", attrs)
 	}
+	if d, err := os.Stat(args.Path); err != nil || !d.IsDir() {
+		return cty.NilVal, errorf("path %s is not a directory", args.Path)
+	}
 	dirname := path.Join(args.Path, block.Labels[1])
 	dir := migrate.OpenMemDir(dirname)
 	// Clear existing directories in case the config was called
@@ -385,8 +410,13 @@ func TemplateDir(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error
 		}
 		return migrate.WriteSumFile(l, sum)
 	})
+	u := fmt.Sprintf("mem://%s", dirname)
+	// Allow using reading the computed dir as a state source.
+	memLoader.states[u] = StateLoaderFunc(func(ctx context.Context, config *StateReaderConfig) (*StateReadCloser, error) {
+		return stateReaderSQL(ctx, config, dir)
+	})
 	return cty.ObjectVal(map[string]cty.Value{
-		"url": cty.StringVal(fmt.Sprintf("mem://%s", dirname)),
+		"url": cty.StringVal(u),
 	}), nil
 }
 
@@ -395,6 +425,7 @@ func TemplateDir(ctx *hcl.EvalContext, block *hclsyntax.Block) (cty.Value, error
 // defined and executed on schemahcl Eval functions.
 type AtlasConfig struct {
 	Client  *cloudapi.Client // Client attached to Atlas Cloud.
+	Token   string           // User token.
 	Project string           // Optional project.
 }
 
@@ -404,7 +435,7 @@ type AtlasConfig struct {
 //	  cloud {
 //	    token   = data.runtimevar.token  // User token.
 //	    url     = var.cloud_url          // Optional URL.
-//	    project = var.project            // Optional project. If set, cloud reporting is enabled.
+//	    project = var.project            // Optional project. Defaults to DefaultProjectName.
 //	  }
 //	}
 func (c *AtlasConfig) InitBlock() schemahcl.Option {
@@ -422,6 +453,7 @@ func (c *AtlasConfig) InitBlock() schemahcl.Option {
 		if args.Cloud.Project == "" {
 			args.Cloud.Project = cloudapi.DefaultProjectName
 		}
+		c.Token = args.Cloud.Token
 		c.Project = args.Cloud.Project
 		c.Client = cloudapi.New(args.Cloud.URL, args.Cloud.Token)
 		cloud := cty.ObjectVal(map[string]cty.Value{
@@ -745,7 +777,7 @@ func (l EntLoader) MigrateDiff(ctx context.Context, opts *MigrateDiffOptions) er
 	if opts.Dev.URL.Schema == "" {
 		return errNotSchemaURL
 	}
-	u, err := url.Parse(opts.To[0])
+	u, err := sqlclient.ParseURL(opts.To[0])
 	if err != nil {
 		return nil
 	}
@@ -790,7 +822,17 @@ func (EntLoader) tables(u *url.URL) ([]*entschema.Table, error) {
 	if err != nil {
 		return nil, err
 	}
-	graph, err := entc.LoadGraph(abs, &gen.Config{})
+	var opts []entc.Option
+	if tags, ok := u.Query()["build-tags"]; ok {
+		opts = append(opts, entc.BuildTags(tags...))
+	}
+	cfg := &gen.Config{}
+	for _, opt := range opts {
+		if err = opt(cfg); err != nil {
+			return nil, err
+		}
+	}
+	graph, err := entc.LoadGraph(abs, cfg)
 	if err != nil {
 		return nil, fmt.Errorf("loading schema: %w", err)
 	}
@@ -813,3 +855,68 @@ func dirFormatter(dir migrate.Dir) migrate.Formatter {
 		return migrate.DefaultFormatter
 	}
 }
+
+func init() {
+	runtimevar.DefaultURLMux().RegisterVariable(Scheme, &URLOpener{})
+}
+
+// Scheme is the URL scheme envvar registers its URLOpener under on runtimevar.DefaultURLMux.
+const Scheme = "env"
+
+// URLOpener opens envvar URLs like "env://VARIABLE".
+type URLOpener struct {
+}
+
+// OpenVariableURL opens the variable at the URL's path. See the package doc
+// for more details.
+func (o *URLOpener) OpenVariableURL(_ context.Context, u *url.URL) (*runtimevar.Variable, error) {
+	return New(os.Getenv(u.Host)), nil
+}
+
+// New constructs a *runtimevar.Variable holding value.
+func New(value string) *runtimevar.Variable {
+	return runtimevar.New(&watcher{value: value, t: time.Now()})
+}
+
+// watcher implements driver.Watcher and driver.State.
+type watcher struct {
+	value string
+	err   error
+	t     time.Time
+}
+
+// Value implements driver.State.Value.
+func (w *watcher) Value() (interface{}, error) {
+	return w.value, w.err
+}
+
+// UpdateTime implements driver.State.UpdateTime.
+func (w *watcher) UpdateTime() time.Time {
+	return w.t
+}
+
+// As implements driver.State.As.
+func (w *watcher) As(any) bool { return false }
+
+// WatchVariable implements driver.WatchVariable.
+func (w *watcher) WatchVariable(ctx context.Context, prev driver.State) (driver.State, time.Duration) {
+	// The first time this is called, return the constant value.
+	if prev == nil {
+		return w, 0
+	}
+	// On subsequent calls, block forever as the value will never change.
+	<-ctx.Done()
+	w.err = ctx.Err()
+	return w, 0
+}
+
+// Close implements driver.Close.
+func (*watcher) Close() error { return nil }
+
+// ErrorAs implements driver.ErrorAs.
+func (*watcher) ErrorAs(error, any) bool { return false }
+
+// ErrorCode implements driver.ErrorCode
+func (*watcher) ErrorCode(error) gcerrors.ErrorCode {
+	return gcerrors.Unknown
+}
