@@ -126,17 +126,18 @@ Last migration attempt had errors:
 
 // MigrateStatus contains a summary of the migration status of a database.
 type MigrateStatus struct {
-	Env       `json:"Env"`
-	Available Files               `json:"Available,omitempty"` // Available migration files
-	Pending   Files               `json:"Pending,omitempty"`   // Pending migration files
-	Applied   []*migrate.Revision `json:"Applied,omitempty"`   // Applied migration files
-	Current   string              `json:"Current,omitempty"`   // Current migration version
-	Next      string              `json:"Next,omitempty"`      // Next migration version
-	Count     int                 `json:"Count,omitempty"`     // Count of applied statements of the last revision
-	Total     int                 `json:"Total,omitempty"`     // Total statements of the last migration
-	Status    string              `json:"Status,omitempty"`    // Status of migration (OK, PENDING)
-	Error     string              `json:"Error,omitempty"`     // Last Error that occurred
-	SQL       string              `json:"SQL,omitempty"`       // SQL that caused the last Error
+	context.Context `json:"-"`
+	Env             `json:"Env"`
+	Available       Files               `json:"Available,omitempty"` // Available migration files
+	Pending         Files               `json:"Pending,omitempty"`   // Pending migration files
+	Applied         []*migrate.Revision `json:"Applied,omitempty"`   // Applied migration files
+	Current         string              `json:"Current,omitempty"`   // Current migration version
+	Next            string              `json:"Next,omitempty"`      // Next migration version
+	Count           int                 `json:"Count,omitempty"`     // Count of applied statements of the last revision
+	Total           int                 `json:"Total,omitempty"`     // Total statements of the last migration
+	Status          string              `json:"Status,omitempty"`    // Status of migration (OK, PENDING)
+	Error           string              `json:"Error,omitempty"`     // Last Error that occurred
+	SQL             string              `json:"SQL,omitempty"`       // SQL that caused the last Error
 }
 
 // Left returns the amount of statements left to apply (if any).
@@ -221,6 +222,7 @@ Current version is {{ cyan .Current.Version }} ({{ .Summary }}):
 type (
 	// MigrateSet contains a summary of the migrate set command.
 	MigrateSet struct {
+		context.Context `json:"-"`
 		// Revisions that were added, removed or updated.
 		Revisions []RevisionOp `json:"Revisions,omitempty"`
 		// Current version in the revisions table.
@@ -326,12 +328,13 @@ type (
 	// MigrateApply contains a summary of a migration applying attempt on a database.
 	MigrateApply struct {
 		Env
-		Pending Files          `json:"Pending,omitempty"` // Pending migration files
-		Applied []*AppliedFile `json:"Applied,omitempty"` // Applied files
-		Current string         `json:"Current,omitempty"` // Current migration version
-		Target  string         `json:"Target,omitempty"`  // Target migration version
-		Start   time.Time
-		End     time.Time
+		context.Context `json:"-"`
+		Pending         Files          `json:"Pending,omitempty"` // Pending migration files
+		Applied         []*AppliedFile `json:"Applied,omitempty"` // Applied files
+		Current         string         `json:"Current,omitempty"` // Current migration version
+		Target          string         `json:"Target,omitempty"`  // Target migration version
+		Start           time.Time
+		End             time.Time
 		// Error is set even then, if it was not caused by a statement in a migration file,
 		// but by Atlas, e.g. when committing or rolling back a transaction.
 		Error string `json:"Error,omitempty"`
@@ -349,10 +352,11 @@ type (
 )
 
 // NewMigrateApply returns an MigrateApply.
-func NewMigrateApply(client *sqlclient.Client, dir migrate.Dir) *MigrateApply {
+func NewMigrateApply(ctx context.Context, client *sqlclient.Client, dir migrate.Dir) *MigrateApply {
 	return &MigrateApply{
-		Env:   NewEnv(client, dir),
-		Start: time.Now(),
+		Context: ctx,
+		Env:     NewEnv(client, dir),
+		Start:   time.Now(),
 	}
 }
 
@@ -471,7 +475,8 @@ type (
 	// SchemaApply contains a summary of a 'schema apply' execution on a database.
 	SchemaApply struct {
 		Env
-		Changes Changes `json:"Changes,omitempty"`
+		context.Context `json:"-"`
+		Changes         Changes `json:"Changes,omitempty"`
 		// General error that occurred during execution.
 		// e.g., when committing or rolling back a transaction.
 		Error string `json:"Error,omitempty"`
@@ -485,9 +490,10 @@ type (
 )
 
 // NewSchemaApply returns a SchemaApply.
-func NewSchemaApply(env Env, applied, pending []*migrate.Change, err *StmtError) *SchemaApply {
+func NewSchemaApply(ctx context.Context, env Env, applied, pending []*migrate.Change, err *StmtError) *SchemaApply {
 	return &SchemaApply{
-		Env: env,
+		Env:     env,
+		Context: ctx,
 		Changes: Changes{
 			Applied: applied,
 			Pending: pending,
@@ -497,8 +503,8 @@ func NewSchemaApply(env Env, applied, pending []*migrate.Change, err *StmtError)
 }
 
 // NewSchemaPlan returns a SchemaApply only with pending changes.
-func NewSchemaPlan(env Env, pending []*migrate.Change, err *StmtError) *SchemaApply {
-	return NewSchemaApply(env, nil, pending, err)
+func NewSchemaPlan(ctx context.Context, env Env, pending []*migrate.Change, err *StmtError) *SchemaApply {
+	return NewSchemaApply(ctx, env, nil, pending, err)
 }
 
 // MarshalJSON implements json.Marshaler.
@@ -520,6 +526,7 @@ func (c Changes) MarshalJSON() ([]byte, error) {
 
 // SchemaInspect contains a summary of the 'schema inspect' command.
 type SchemaInspect struct {
+	context.Context   `json:"-"`
 	*sqlclient.Client `json:"-"`
 	Realm             *schema.Realm `json:"Schema,omitempty"` // Inspected realm.
 }
@@ -550,10 +557,16 @@ func (s *SchemaInspect) MarshalHCL() (string, error) {
 // MarshalJSON implements json.Marshaler.
 func (s *SchemaInspect) MarshalJSON() ([]byte, error) {
 	type (
+		Attrs struct {
+			Comment string `json:"comment,omitempty"`
+			Charset string `json:"charset,omitempty"`
+			Collate string `json:"collate,omitempty"`
+		}
 		Column struct {
 			Name string `json:"name"`
 			Type string `json:"type,omitempty"`
 			Null bool   `json:"null,omitempty"`
+			Attrs
 		}
 		IndexPart struct {
 			Desc   bool   `json:"desc,omitempty"`
@@ -579,25 +592,45 @@ func (s *SchemaInspect) MarshalJSON() ([]byte, error) {
 			Indexes     []Index      `json:"indexes,omitempty"`
 			PrimaryKey  *Index       `json:"primary_key,omitempty"`
 			ForeignKeys []ForeignKey `json:"foreign_keys,omitempty"`
+			Attrs
 		}
 		Schema struct {
 			Name   string  `json:"name"`
 			Tables []Table `json:"tables,omitempty"`
+			Attrs
+		}
+	)
+	var (
+		realm struct {
+			Schemas []Schema `json:"schemas,omitempty"`
+		}
+		setAttrs = func(from []schema.Attr, to *Attrs) {
+			for i := range from {
+				switch a := from[i].(type) {
+				case *schema.Comment:
+					to.Comment = a.Text
+				case *schema.Charset:
+					to.Charset = a.V
+				case *schema.Collation:
+					to.Collate = a.V
+				}
+			}
 		}
 	)
-	var realm struct {
-		Schemas []Schema `json:"schemas,omitempty"`
-	}
 	for _, s1 := range s.Realm.Schemas {
 		s2 := Schema{Name: s1.Name}
+		setAttrs(s1.Attrs, &s2.Attrs)
 		for _, t1 := range s1.Tables {
 			t2 := Table{Name: t1.Name}
+			setAttrs(t1.Attrs, &t2.Attrs)
 			for _, c1 := range t1.Columns {
-				t2.Columns = append(t2.Columns, Column{
+				c2 := Column{
 					Name: c1.Name,
 					Type: c1.Type.Raw,
 					Null: c1.Type.Null,
-				})
+				}
+				setAttrs(c1.Attrs, &c2.Attrs)
+				t2.Columns = append(t2.Columns, c2)
 			}
 			idxParts := func(idx *schema.Index) (parts []IndexPart) {
 				for _, p1 := range idx.Parts {
@@ -666,15 +699,22 @@ func sqlInspect(report *SchemaInspect, indent ...string) (string, error) {
 		for _, o := range s.Objects {
 			changes = append(changes, &schema.AddObject{O: o})
 		}
+		for _, f := range s.Funcs {
+			changes = append(changes, &schema.AddFunc{F: f})
+		}
+		for _, p := range s.Procs {
+			changes = append(changes, &schema.AddProc{P: p})
+		}
 	}
-	return fmtPlan(report.Client, changes, indent)
+	return fmtPlan(report.Context, report.Client, changes, indent)
 }
 
 // SchemaDiff contains a summary of the 'schema diff' command.
 type SchemaDiff struct {
-	*sqlclient.Client
-	From, To *schema.Realm
-	Changes  []schema.Change
+	context.Context   `json:"-"`
+	*sqlclient.Client `json:"-"`
+	From, To          *schema.Realm
+	Changes           []schema.Change
 }
 
 var (
@@ -700,14 +740,14 @@ func (s *SchemaDiff) MarshalSQL(indent ...string) (string, error) {
 }
 
 func sqlDiff(diff *SchemaDiff, indent ...string) (string, error) {
-	return fmtPlan(diff.Client, diff.Changes, indent)
+	return fmtPlan(diff.Context, diff.Client, diff.Changes, indent)
 }
 
-func fmtPlan(client *sqlclient.Client, changes schema.Changes, indent []string) (string, error) {
+func fmtPlan(ctx context.Context, client *sqlclient.Client, changes schema.Changes, indent []string) (string, error) {
 	if len(indent) > 1 {
 		return "", fmt.Errorf("unexpected number of arguments: %d", len(indent))
 	}
-	plan, err := client.PlanChanges(context.Background(), "plan", changes, func(o *migrate.PlanOptions) {
+	plan, err := client.PlanChanges(ctx, "plan", changes, func(o *migrate.PlanOptions) {
 		o.Mode = migrate.PlanModeDump
 		// Disable tables qualifier in schema-mode.
 		if client.URL.Schema != "" {
