@@ -9,7 +9,7 @@ module.exports = {
   organizationName: 'ariga',
   projectName: 'atlas',
   themeConfig: {
-    image: "https://atlas-og-img.vercel.app/**Atlas%20%7C**%20Open-source%20database%20schema%20management.png?theme=dark",
+    image: "https://og.atlasgo.io/image?title=Schema%20Migration%20Tool%20for%20any%20Language",
     prism: {
       additionalLanguages: ['hcl'],
       magicComments: [
@@ -58,7 +58,7 @@ module.exports = {
           className: 'header-newsletter-link',
           position: 'right',
         },
-        
+
         {
           to: 'getting-started',
           label: 'Docs',
@@ -140,8 +140,8 @@ module.exports = {
       `,
     },
     announcementBar: {
-      id: 'announcementBar-6', // Increment on change
-      content: `<strong>Announcing Atlas v0.13: SQL Views, MSSQL, ERD, and more. </strong><a class="cta" href="https://atlasgo.io/blog/2023/08/06/atlas-v-0-13">Read it here</a>`,
+      id: 'announcementBar-7', // Increment on change
+      content: `<strong>Announcing Atlas v0.14: Checkpoints, Push to Cloud, and JetBrains support. </strong><a class="cta" href="https://atlasgo.io/blog/2023/08/31/atlas-v-0-14">Read it here</a>`,
       isCloseable: true,
       backgroundColor: '#008CFA',
       textColor: '#fff',
@@ -231,6 +231,14 @@ module.exports = {
           {
             from: '/integrations/go-api',
             to: '/integrations/go-sdk',
+          },
+          {
+            from: '/deployment/intro',
+            to: '/cloud/deployment',
+          },
+          {
+            from: '/ui/intro',
+            to: '/cloud/getting-started',
           }
         ],
       },
