@@ -1520,7 +1520,7 @@ func (t *pgTest) valueByVersion(values map[string]string, defaults string) strin
 func (t *pgTest) loadRealm() *schema.Realm {
 	r, err := t.drv.InspectRealm(context.Background(), &schema.InspectRealmOption{
 		Schemas: []string{"public"},
-		Mode:    ^schema.InspectViews,
+		Mode:    schema.InspectSchemas | schema.InspectTables,
 	})
 	require.NoError(t, err)
 	return r
@@ -1611,10 +1611,6 @@ func (t *pgTest) realm() *schema.Realm {
 				},
 			},
 		},
-		Attrs: []schema.Attr{
-			&schema.Collation{V: "en_US.utf8"},
-			&postgres.CType{V: "en_US.utf8"},
-		},
 	}
 	r.Schemas[0].Realm = r
 	return r
