@@ -91,6 +91,15 @@ type (
 	}
 )
 
+// IsRefTo indicates if the Type is a reference to specific schema type definition.
+func (t *Type) IsRefTo(n string) bool {
+	if !t.IsRef {
+		return false
+	}
+	path, err := (&Ref{V: t.T}).ByType(n)
+	return err == nil && len(path) > 0
+}
+
 // IsRef indicates if the attribute is a reference type.
 func (a *Attr) IsRef() bool {
 	if !a.V.Type().IsCapsuleType() {
@@ -460,6 +469,16 @@ func RefAttr(k string, v *Ref) *Attr {
 	}
 }
 
+// RefValue is a helper method for constructing a cty.Value that contains a Ref value.
+func RefValue(v string) cty.Value {
+	return cty.CapsuleVal(ctyRefType, &Ref{V: v})
+}
+
+// TypeValue is a helper method for constructing a cty.Value that contains a Type value.
+func TypeValue(t *Type) cty.Value {
+	return cty.CapsuleVal(ctyTypeSpec, t)
+}
+
 // StringsAttr is a helper method for constructing *schemahcl.Attr instances that contain list strings.
 func StringsAttr(k string, vs ...string) *Attr {
 	vv := make([]cty.Value, len(vs))
