@@ -26,6 +26,7 @@ type (
 	ConvertTypeFunc        func(*sqlspec.Column) (schema.Type, error)
 	ConvertPrimaryKeyFunc  func(*sqlspec.PrimaryKey, *schema.Table) (*schema.Index, error)
 	ConvertIndexFunc       func(*sqlspec.Index, *schema.Table) (*schema.Index, error)
+	ConvertViewIndexFunc   func(*sqlspec.Index, *schema.View) (*schema.Index, error)
 	ConvertCheckFunc       func(*sqlspec.Check) (*schema.Check, error)
 	ColumnTypeSpecFunc     func(schema.Type) (*sqlspec.Column, error)
 	TableSpecFunc          func(*schema.Table) (*sqlspec.Table, error)
@@ -41,16 +42,39 @@ type (
 type (
 	// ScanDoc represents a scanned HCL document.
 	ScanDoc struct {
-		Schemas []*sqlspec.Schema
-		Tables  []*sqlspec.Table
-		Views   []*sqlspec.View
+		Schemas      []*sqlspec.Schema
+		Tables       []*sqlspec.Table
+		Views        []*sqlspec.View
+		Materialized []*sqlspec.View
+		Funcs        []*sqlspec.Func
+		Procs        []*sqlspec.Func
 	}
+
 	// ScanFuncs represents a set of scan functions
 	// used to convert the HCL document to the Realm.
 	ScanFuncs struct {
 		Table ConvertTableFunc
 		View  ConvertViewFunc
+		Func  func(*sqlspec.Func) (*schema.Func, error)
+		Proc  func(*sqlspec.Func) (*schema.Proc, error)
 	}
+
+	// Funcs represents a set of spec functions
+	// used to convert the Realm to an HCL document.
+	Funcs struct {
+		Table TableSpecFunc
+		View  ViewSpecFunc
+		Func  func(*schema.Func) (*sqlspec.Func, error)
+		Proc  func(*schema.Proc) (*sqlspec.Func, error)
+	}
+)
+
+const (
+	typeView         = "view"
+	typeTable        = "table"
+	typeColumn       = "column"
+	typeSchema       = "schema"
+	typeMaterialized = "materialized"
 )
 
 // Scan populates the Realm from the schemas and table specs.
@@ -110,41 +134,113 @@ func Scan(r *schema.Realm, doc *ScanDoc, funcs *ScanFuncs) error {
 			viewDeps[v] = refs
 		}
 	}
+	for _, m := range doc.Materialized {
+		name, err := SchemaName(m.Schema)
+		if err != nil {
+			return fmt.Errorf("specutil: cannot extract schema name for materialized %q: %w", m.Name, err)
+		}
+		s, ok := byName[name]
+		if !ok {
+			return fmt.Errorf("specutil: schema %q not found for materialized %q", name, m.Name)
+		}
+		v, err := funcs.View(m, s)
+		if err != nil {
+			return fmt.Errorf("specutil: cannot convert materialized %q: %w", m.Name, err)
+		}
+		s.AddViews(v.SetMaterialized(true))
+		if deps, ok := m.Attr("depends_on"); ok {
+			refs, err := deps.Refs()
+			if err != nil {
+				return fmt.Errorf("specutil: expect list of references for attribute materialized.%s.depends_on: %w", m.Name, err)
+			}
+			viewDeps[v] = refs
+		}
+	}
 	// Link views' dependencies.
 	for v, refs := range viewDeps {
+		srcT := typeView
+		if v.Materialized() {
+			srcT = typeMaterialized
+		}
 		for i, r := range refs {
 			switch p, err := r.Path(); {
 			case err != nil:
-				return fmt.Errorf("specutil: extract reference for view.%s: %w", v.Name, err)
+				return fmt.Errorf("specutil: extract reference for %s.%s: %w", srcT, v.Name, err)
 			case len(p) == 0:
-				return fmt.Errorf("specutil: empty reference for view.%s", v.Name)
-			case p[0].T == "view":
-				q, n, err := viewName(r)
+				return fmt.Errorf("specutil: empty reference for %s.%s", srcT, v.Name)
+			case p[0].T == typeView:
+				q, n, err := RefName(r, typeView)
 				if err != nil {
-					return fmt.Errorf("specutil: extract view name from view.%s.depends_on[%d]: %w", v.Name, i, err)
+					return fmt.Errorf("specutil: extract view name from %s.%s.depends_on[%d]: %w", srcT, v.Name, i, err)
 				}
 				v1, err := findT(v.Schema, q, n, func(s *schema.Schema, name string) (*schema.View, bool) {
 					return s.View(name)
 				})
 				if err != nil {
-					return fmt.Errorf("specutil: find view refrence for view.%s.depends_on[%d]: %w", v.Name, i, err)
+					return fmt.Errorf("specutil: find view refrence for %s.%s.depends_on[%d]: %w", srcT, v.Name, i, err)
 				}
 				v.AddDeps(v1)
-			case p[0].T == "table":
-				q, n, err := tableName(r)
+			case p[0].T == typeMaterialized:
+				q, n, err := RefName(r, typeMaterialized)
 				if err != nil {
-					return fmt.Errorf("specutil: extract view name from view.%s.depends_on[%d]: %w", v.Name, i, err)
+					return fmt.Errorf("specutil: extract materialized name from %s.%s.depends_on[%d]: %w", srcT, v.Name, i, err)
+				}
+				v1, err := findT(v.Schema, q, n, func(s *schema.Schema, name string) (*schema.View, bool) {
+					return s.Materialized(name)
+				})
+				if err != nil {
+					return fmt.Errorf("specutil: find materialized refrence for %s.%s.depends_on[%d]: %w", srcT, v.Name, i, err)
+				}
+				v.AddDeps(v1)
+			case p[0].T == typeTable:
+				q, n, err := RefName(r, typeTable)
+				if err != nil {
+					return fmt.Errorf("specutil: extract table name from %s.%s.depends_on[%d]: %w", srcT, v.Name, i, err)
 				}
 				t1, err := findT(v.Schema, q, n, func(s *schema.Schema, name string) (*schema.Table, bool) {
 					return s.Table(name)
 				})
 				if err != nil {
-					return fmt.Errorf("specutil: find table refrence for view.%s.depends_on[%d]: %w", v.Name, i, err)
+					return fmt.Errorf("specutil: find table refrence for %s.%s.depends_on[%d]: %w", srcT, v.Name, i, err)
 				}
 				v.AddDeps(t1)
 			}
 		}
 	}
+	if funcs.Func != nil {
+		for _, sf := range doc.Funcs {
+			name, err := SchemaName(sf.Schema)
+			if err != nil {
+				return fmt.Errorf("specutil: cannot extract schema name for function %q: %w", sf.Name, err)
+			}
+			s, ok := byName[name]
+			if !ok {
+				return fmt.Errorf("specutil: schema %q not found for function %q", name, sf.Name)
+			}
+			f, err := funcs.Func(sf)
+			if err != nil {
+				return fmt.Errorf("specutil: cannot convert function %q: %w", sf.Name, err)
+			}
+			s.AddFuncs(f)
+		}
+	}
+	if funcs.Proc != nil {
+		for _, sf := range doc.Procs {
+			name, err := SchemaName(sf.Schema)
+			if err != nil {
+				return fmt.Errorf("specutil: cannot extract schema name for procedure %q: %w", sf.Name, err)
+			}
+			s, ok := byName[name]
+			if !ok {
+				return fmt.Errorf("specutil: schema %q not found for procedure %q", name, sf.Name)
+			}
+			f, err := funcs.Proc(sf)
+			if err != nil {
+				return fmt.Errorf("specutil: cannot convert procedure %q: %w", sf.Name, err)
+			}
+			s.AddProcs(f)
+		}
+	}
 	return nil
 }
 
@@ -153,46 +249,46 @@ func Scan(r *schema.Realm, doc *ScanDoc, funcs *ScanFuncs) error {
 // at this point. Instead, the linking is done by the Schema function.
 func Table(spec *sqlspec.Table, parent *schema.Schema, convertColumn ConvertTableColumnFunc,
 	convertPK ConvertPrimaryKeyFunc, convertIndex ConvertIndexFunc, convertCheck ConvertCheckFunc) (*schema.Table, error) {
-	tbl := &schema.Table{
+	t := &schema.Table{
 		Name:   spec.Name,
 		Schema: parent,
 	}
 	for _, csp := range spec.Columns {
-		col, err := convertColumn(csp, tbl)
+		col, err := convertColumn(csp, t)
 		if err != nil {
 			return nil, err
 		}
-		tbl.Columns = append(tbl.Columns, col)
+		t.AddColumns(col)
 	}
 	if spec.PrimaryKey != nil {
-		pk, err := convertPK(spec.PrimaryKey, tbl)
+		pk, err := convertPK(spec.PrimaryKey, t)
 		if err != nil {
 			return nil, err
 		}
-		tbl.PrimaryKey = pk
+		t.SetPrimaryKey(pk)
 	}
 	for _, idx := range spec.Indexes {
-		i, err := convertIndex(idx, tbl)
+		i, err := convertIndex(idx, t)
 		if err != nil {
 			return nil, err
 		}
-		tbl.Indexes = append(tbl.Indexes, i)
+		t.AddIndexes(i)
 	}
 	for _, c := range spec.Checks {
 		c, err := convertCheck(c)
 		if err != nil {
 			return nil, err
 		}
-		tbl.AddChecks(c)
+		t.AddChecks(c)
 	}
-	if err := convertCommentFromSpec(spec, &tbl.Attrs); err != nil {
+	if err := convertCommentFromSpec(spec, &t.Attrs); err != nil {
 		return nil, err
 	}
-	return tbl, nil
+	return t, nil
 }
 
 // View converts a sqlspec.View to a schema.View.
-func View(spec *sqlspec.View, parent *schema.Schema, convertColumn ConvertViewColumnFunc) (*schema.View, error) {
+func View(spec *sqlspec.View, parent *schema.Schema, convertC ConvertViewColumnFunc, convertI ConvertViewIndexFunc) (*schema.View, error) {
 	as, ok := spec.Extra.Attr("as")
 	if !ok {
 		return nil, fmt.Errorf("specutil: missing 'as' definition for view %q", spec.Name)
@@ -203,12 +299,19 @@ func View(spec *sqlspec.View, parent *schema.Schema, convertColumn ConvertViewCo
 	}
 	v := schema.NewView(spec.Name, def).SetSchema(parent)
 	for _, c := range spec.Columns {
-		c, err := convertColumn(c, v)
+		c, err := convertC(c, v)
 		if err != nil {
 			return nil, err
 		}
 		v.AddColumns(c)
 	}
+	for _, idx := range spec.Indexes {
+		i, err := convertI(idx, v)
+		if err != nil {
+			return nil, err
+		}
+		v.AddIndexes(i)
+	}
 	if err := convertCommentFromSpec(spec, &v.Attrs); err != nil {
 		return nil, err
 	}
@@ -230,24 +333,11 @@ func Column(spec *sqlspec.Column, conv ConvertTypeFunc) (*schema.Column, error)
 			Null: spec.Null,
 		},
 	}
-	if d := spec.Default; !d.IsNull() {
-		switch {
-		case d.Type() == cty.String:
-			out.Default = &schema.Literal{V: d.AsString()}
-		case d.Type() == cty.Number:
-			out.Default = &schema.Literal{V: d.AsBigFloat().String()}
-		case d.Type() == cty.Bool:
-			out.Default = &schema.Literal{V: strconv.FormatBool(d.True())}
-		case d.Type().IsCapsuleType():
-			x, ok := d.EncapsulatedValue().(*schemahcl.RawExpr)
-			if !ok {
-				return nil, fmt.Errorf("invalid default value %q", d.Type().FriendlyName())
-			}
-			out.Default = &schema.RawExpr{X: x.X}
-		default:
-			return nil, fmt.Errorf("unsupported value type for default: %T", d)
-		}
+	d, err := Default(spec.Default)
+	if err != nil {
+		return nil, err
 	}
+	out.Default = d
 	ct, err := conv(spec)
 	if err != nil {
 		return nil, err
@@ -259,6 +349,31 @@ func Column(spec *sqlspec.Column, conv ConvertTypeFunc) (*schema.Column, error)
 	return out, err
 }
 
+// Default converts a cty.Value (as defined in the spec) into a schema.Expr.
+func Default(d cty.Value) (schema.Expr, error) {
+	if d.IsNull() {
+		return nil, nil // no default.
+	}
+	var x schema.Expr
+	switch {
+	case d.Type() == cty.String:
+		x = &schema.Literal{V: d.AsString()}
+	case d.Type() == cty.Number:
+		x = &schema.Literal{V: d.AsBigFloat().String()}
+	case d.Type() == cty.Bool:
+		x = &schema.Literal{V: strconv.FormatBool(d.True())}
+	case d.Type().IsCapsuleType():
+		raw, ok := d.EncapsulatedValue().(*schemahcl.RawExpr)
+		if !ok {
+			return nil, fmt.Errorf("invalid default value %q", d.Type().FriendlyName())
+		}
+		x = &schema.RawExpr{X: raw.X}
+	default:
+		return nil, fmt.Errorf("unsupported value type for default: %T", d)
+	}
+	return x, nil
+}
+
 // Index converts a sqlspec.Index to a schema.Index. The optional arguments allow
 // passing functions for mutating the created index-part (e.g. add attributes).
 func Index(spec *sqlspec.Index, parent *schema.Table, partFns ...func(*sqlspec.IndexPart, *schema.IndexPart) error) (*schema.Index, error) {
@@ -386,40 +501,65 @@ func linkForeignKeys(tbl *schema.Table, fks []*sqlspec.ForeignKey) error {
 }
 
 // FromSchema converts a schema.Schema into sqlspec.Schema and []sqlspec.Table.
-func FromSchema(s *schema.Schema, specT TableSpecFunc, specV ViewSpecFunc) (*SchemaSpec, error) {
-	var (
-		spec = &sqlspec.Schema{
+func FromSchema(s *schema.Schema, funcs *Funcs) (*SchemaSpec, error) {
+	spec := &SchemaSpec{
+		Schema: &sqlspec.Schema{
 			Name: s.Name,
-		}
-		views  = make([]*sqlspec.View, 0, len(s.Views))
-		tables = make([]*sqlspec.Table, 0, len(s.Tables))
-	)
+		},
+		Tables:       make([]*sqlspec.Table, 0, len(s.Tables)),
+		Views:        make([]*sqlspec.View, 0, len(s.Views)),
+		Materialized: make([]*sqlspec.View, 0, len(s.Views)),
+	}
 	for _, t := range s.Tables {
-		table, err := specT(t)
+		table, err := funcs.Table(t)
 		if err != nil {
 			return nil, err
 		}
 		if s.Name != "" {
 			table.Schema = SchemaRef(s.Name)
 		}
-		tables = append(tables, table)
+		spec.Tables = append(spec.Tables, table)
 	}
 	for _, v := range s.Views {
-		view, err := specV(v)
+		view, err := funcs.View(v)
 		if err != nil {
 			return nil, err
 		}
 		if s.Name != "" {
 			view.Schema = SchemaRef(s.Name)
 		}
-		views = append(views, view)
+		if v.Materialized() {
+			spec.Materialized = append(spec.Materialized, view)
+		} else {
+			spec.Views = append(spec.Views, view)
+		}
 	}
-	convertCommentFromSchema(s.Attrs, &spec.Extra.Attrs)
-	return &SchemaSpec{
-		Schema: spec,
-		Tables: tables,
-		Views:  views,
-	}, nil
+	if funcs.Func != nil {
+		for _, f := range s.Funcs {
+			fn, err := funcs.Func(f)
+			if err != nil {
+				return nil, err
+			}
+			if s.Name != "" {
+				fn.Schema = SchemaRef(s.Name)
+			}
+			spec.Funcs = append(spec.Funcs, fn)
+		}
+	}
+	if funcs.Proc != nil {
+		for _, p := range s.Procs {
+			pr, err := funcs.Proc(p)
+			if err != nil {
+				return nil, err
+			}
+			if s.Name != "" {
+				pr.Schema = SchemaRef(s.Name)
+			}
+			spec.Procs = append(spec.Procs, pr)
+		}
+	}
+	convertCommentFromSchema(s.Attrs, &spec.Schema.Extra.Attrs)
+	return spec, nil
 }
 
 // FromTable converts a schema.Table to a sqlspec.Table.
@@ -466,7 +606,7 @@ func FromTable(t *schema.Table, colFn TableColumnSpecFunc, pkFn PrimaryKeySpecFu
 }
 
 // FromView converts a schema.View to a sqlspec.View.
-func FromView(v *schema.View, colFn ViewColumnSpecFunc) (*sqlspec.View, error) {
+func FromView(v *schema.View, colFn ViewColumnSpecFunc, idxFn IndexSpecFunc) (*sqlspec.View, error) {
 	spec := &sqlspec.View{
 		Name: v.Name,
 	}
@@ -477,6 +617,13 @@ func FromView(v *schema.View, colFn ViewColumnSpecFunc) (*sqlspec.View, error) {
 		}
 		spec.Columns = append(spec.Columns, cs)
 	}
+	for _, idx := range v.Indexes {
+		i, err := idxFn(idx)
+		if err != nil {
+			return nil, err
+		}
+		spec.Indexes = append(spec.Indexes, i)
+	}
 	as := v.Def
 	// In case the view definition is multi-line,
 	// format it as indented heredoc with two spaces.
@@ -521,14 +668,18 @@ func FromView(v *schema.View, colFn ViewColumnSpecFunc) (*sqlspec.View, error) {
 				path = append(path, d.Schema.Name)
 			}
 			deps = append(deps, schemahcl.BuildRef([]schemahcl.PathIndex{
-				{T: "table", V: append(path, d.Name)},
+				{T: typeTable, V: append(path, d.Name)},
 			}))
 		case *schema.View:
+			vt := typeView
+			if d.Materialized() {
+				vt = typeMaterialized
+			}
 			if nameV[d.Name] > 1 {
 				path = append(path, d.Schema.Name)
 			}
 			deps = append(deps, schemahcl.BuildRef([]schemahcl.PathIndex{
-				{T: "view", V: append(path, d.Name)},
+				{T: vt, V: append(path, d.Name)},
 			}))
 		}
 	}
@@ -743,7 +894,7 @@ func FromCheck(s *schema.Check) *sqlspec.Check {
 
 // SchemaName returns the name from a ref to a schema.
 func SchemaName(ref *schemahcl.Ref) (string, error) {
-	vs, err := ref.ByType("schema")
+	vs, err := ref.ByType(typeSchema)
 	if err != nil {
 		return "", err
 	}
@@ -755,7 +906,7 @@ func SchemaName(ref *schemahcl.Ref) (string, error) {
 
 // ColumnByRef returns a column from the table by its reference.
 func ColumnByRef(t *schema.Table, ref *schemahcl.Ref) (*schema.Column, error) {
-	vs, err := ref.ByType("column")
+	vs, err := ref.ByType(typeColumn)
 	if err != nil {
 		return nil, err
 	}
@@ -822,24 +973,13 @@ func findT[T schema.View | schema.Table](sch *schema.Schema, qualifier, name str
 	}
 }
 
-func tableName(ref *schemahcl.Ref) (qualifier, name string, err error) {
-	vs, err := ref.ByType("table")
-	if err != nil {
-		return "", "", err
-	}
-	switch len(vs) {
-	case 1:
-		name = vs[0]
-	case 2:
-		qualifier, name = vs[0], vs[1]
-	default:
-		return "", "", fmt.Errorf("sqlspec: unexpected number of references in %q", vs)
-	}
-	return
+func tableName(ref *schemahcl.Ref) (string, string, error) {
+	return RefName(ref, typeTable)
 }
 
-func viewName(ref *schemahcl.Ref) (qualifier, name string, err error) {
-	vs, err := ref.ByType("view")
+// RefName returns the qualifier and name from a reference.
+func RefName(ref *schemahcl.Ref, typeName string) (qualifier, name string, err error) {
+	vs, err := ref.ByType(typeName)
 	if err != nil {
 		return "", "", err
 	}
@@ -861,28 +1001,28 @@ func isLocalRef(r *schemahcl.Ref) bool {
 // ColumnRef returns the reference of a column by its name.
 func ColumnRef(cName string) *schemahcl.Ref {
 	return schemahcl.BuildRef([]schemahcl.PathIndex{
-		{T: "column", V: []string{cName}},
+		{T: typeColumn, V: []string{cName}},
 	})
 }
 
 func externalColRef(cName string, tName string) *schemahcl.Ref {
 	return schemahcl.BuildRef([]schemahcl.PathIndex{
-		{T: "table", V: []string{tName}},
-		{T: "column", V: []string{cName}},
+		{T: typeTable, V: []string{tName}},
+		{T: typeColumn, V: []string{cName}},
 	})
 }
 
 func qualifiedExternalColRef(cName, tName, sName string) *schemahcl.Ref {
 	return schemahcl.BuildRef([]schemahcl.PathIndex{
-		{T: "table", V: []string{sName, tName}},
-		{T: "column", V: []string{cName}},
+		{T: typeTable, V: []string{sName, tName}},
+		{T: typeColumn, V: []string{cName}},
 	})
 }
 
 // SchemaRef returns the schemahcl.Ref to the schema with the given name.
 func SchemaRef(name string) *schemahcl.Ref {
 	return schemahcl.BuildRef([]schemahcl.PathIndex{
-		{T: "schema", V: []string{name}},
+		{T: typeSchema, V: []string{name}},
 	})
 }
 
