@@ -62,12 +62,12 @@ func Stmts(input string) ([]*Stmt, error) {
 }
 
 type lex struct {
-	input    string
-	pos      int      // current phase position
-	total    int      // total bytes scanned so far
-	width    int      // size of latest rune
-	delim    string   // configured delimiter
-	comments []string // collected comments
+	src, input string   // src and current input text
+	pos        int      // current phase position
+	total      int      // total bytes scanned so far
+	width      int      // size of latest rune
+	delim      string   // configured delimiter
+	comments   []string // collected comments
 }
 
 const (
@@ -77,7 +77,7 @@ const (
 )
 
 func newLex(input string) (*lex, error) {
-	l := &lex{input: input, delim: delimiter}
+	l := &lex{src: input, input: input, delim: delimiter}
 	if d, ok := directive(input, directiveDelimiter, directivePrefixSQL); ok {
 		if err := l.setDelim(d); err != nil {
 			return nil, err
@@ -91,8 +91,13 @@ func newLex(input string) (*lex, error) {
 	return l, nil
 }
 
-// Dollar-quoted string as defined by the PostgreSQL scanner.
-var reDollarQuote = regexp.MustCompile(`^\$([A-Za-zÈ-ÿ_][\wÈ-ÿ]*)*\$`)
+var (
+	// Dollar-quoted string as defined by the PostgreSQL scanner.
+	reDollarQuote = regexp.MustCompile(`^\$([A-Za-zÈ-ÿ_][\wÈ-ÿ]*)*\$`)
+	// The 'BEGIN ATOMIC' syntax as specified in the SQL 2003 standard.
+	reBeginAtomic = regexp.MustCompile(`(?i)^\s*BEGIN\s+ATOMIC\s+`)
+	reEnd         = regexp.MustCompile(`(?i)^\s*END\s*`)
+)
 
 func (l *lex) stmt() (*Stmt, error) {
 	var (
@@ -143,12 +148,20 @@ Scan:
 			if err := l.skipDollarQuote(); err != nil {
 				return nil, err
 			}
-		case r == '#':
+		// Skip non-standard MySQL comments if they are inside
+		// expressions until we make the lexer driver-aware.
+		case depth == 0 && r == '#':
 			l.comment("#", "\n")
 		case r == '-' && l.next() == '-':
 			l.comment("--", "\n")
 		case r == '/' && l.next() == '*':
 			l.comment("/*", "*/")
+		case reBeginAtomic.MatchString(l.input[l.pos-1:]):
+			if err := l.skipBeginAtomic(); err != nil {
+				return nil, err
+			}
+			text = l.input[:l.pos]
+			break Scan
 		}
 	}
 	return l.emit(text), nil
@@ -211,6 +224,32 @@ func (l *lex) skipDollarQuote() error {
 	}
 }
 
+func (l *lex) skipBeginAtomic() error {
+	m := reBeginAtomic.FindString(l.input[l.pos-1:])
+	if m == "" {
+		return l.error(l.pos, "unexpected missing BEGIN ATOMIC block")
+	}
+	l.addPos(len(m) - 1)
+	body, err := newLex(l.input[l.pos:])
+	if err != nil {
+		return l.error(l.pos, "create scanner for sql body: %v", err)
+	}
+	for {
+		s, err := body.stmt()
+		if err == io.EOF {
+			return l.error(l.pos, "unexpected eof when scanning sql body")
+		}
+		if err != nil {
+			return l.error(l.pos, "scan sql body: %v", err)
+		}
+		if reEnd.MatchString(s.Text) {
+			break
+		}
+	}
+	l.addPos(body.total)
+	return nil
+}
+
 func (l *lex) comment(left, right string) {
 	i := strings.Index(l.input[l.pos:], right)
 	// Not a comment.
@@ -289,14 +328,15 @@ func (l *lex) setDelim(d string) error {
 func (l *lex) error(pos int, format string, args ...any) error {
 	format = "%d:%d: " + format
 	var (
-		s    = l.input[:pos]
+		p    = len(l.src) - len(l.input) + pos
+		s    = l.src[:p]
 		col  = strings.LastIndex(s, "\n")
 		line = 1 + strings.Count(s, "\n")
 	)
 	if line == 1 {
-		col = pos
+		col = p
 	} else {
-		col = pos - col - 1
+		col = p - col - 1
 	}
 	return fmt.Errorf(format, append([]any{line, col}, args...)...)
 }
