@@ -44,6 +44,11 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 				return nil, err
 			}
 		}
+		if mode.Is(schema.InspectFuncs) {
+			if err := i.inspectFuncs(ctx, r, nil); err != nil {
+				return nil, err
+			}
+		}
 	}
 	return sqlx.ExcludeRealm(r, opts.Exclude)
 }
@@ -76,6 +81,11 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 			return nil, err
 		}
 	}
+	if sqlx.ModeInspectSchema(opts).Is(schema.InspectFuncs) {
+		if err := i.inspectFuncs(ctx, r, opts); err != nil {
+			return nil, err
+		}
+	}
 	return sqlx.ExcludeSchema(r.Schemas[0], opts.Exclude)
 }
 
@@ -262,7 +272,7 @@ func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) error {
 	}
 	ct, err := ParseType(c.Type.Raw)
 	if err != nil {
-		return err
+		return fmt.Errorf("parse %q.%q type %q: %w", t.Name, c.Name, c.Type.Raw, err)
 	}
 	c.Type.Type = ct
 	attr, err := parseExtra(extra.String)
@@ -451,6 +461,9 @@ func (i *inspect) checks(ctx context.Context, s *schema.Schema) error {
 				// Unset the inspected CHARSET/COLLATE attributes
 				// as they are valid only for character types.
 				c.UnsetCharset().UnsetCollation()
+				// Skip adding this CHECK constraint to the table definition
+				// as it is implicitly created by MariaDB for this JSON column.
+				continue
 			}
 		} else if enforced.String == "NO" {
 			// The ENFORCED attribute is not supported by MariaDB.
@@ -589,6 +602,11 @@ func (i *inspect) myDefaultExpr(c *schema.Column, x string, attr *extraAttr) sch
 
 // parseColumn returns column parts, size and signed-info from a MySQL type.
 func parseColumn(typ string) (parts []string, size int, unsigned bool, err error) {
+	// Remove MariaDB like comments embedded in the type
+	// for compatibility. For example: /* mariadb-5.3 */.
+	if i := strings.Index(typ, "/*"); i > 0 && strings.HasSuffix(strings.TrimSpace(typ), "*/") {
+		typ = strings.TrimSpace(typ[:i])
+	}
 	switch parts = strings.FieldsFunc(typ, func(r rune) bool {
 		return r == '(' || r == ')' || r == ' ' || r == ','
 	}); parts[0] {
