@@ -24,6 +24,8 @@ type doc struct {
 	Tables  []*sqlspec.Table  `spec:"table"`
 	Views   []*sqlspec.View   `spec:"view"`
 	Schemas []*sqlspec.Schema `spec:"schema"`
+	Funcs   []*sqlspec.Func   `spec:"function"`
+	Procs   []*sqlspec.Func   `spec:"procedure"`
 }
 
 // evalSpec evaluates an Atlas DDL document into v using the input.
@@ -35,8 +37,8 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 			return err
 		}
 		if err := specutil.Scan(v,
-			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views},
-			&specutil.ScanFuncs{Table: convertTable, View: convertView},
+			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views, Funcs: d.Funcs, Procs: d.Procs},
+			scanFuncs,
 		); err != nil {
 			return fmt.Errorf("mysql: failed converting to *schema.Realm: %w", err)
 		}
@@ -59,8 +61,8 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		}
 		r := &schema.Realm{}
 		if err := specutil.Scan(r,
-			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views},
-			&specutil.ScanFuncs{Table: convertTable, View: convertView},
+			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views, Funcs: d.Funcs, Procs: d.Procs},
+			scanFuncs,
 		); err != nil {
 			return err
 		}
@@ -141,9 +143,15 @@ func convertTable(spec *sqlspec.Table, parent *schema.Schema) (*schema.Table, er
 
 // convertView converts a sqlspec.View to a schema.View.
 func convertView(spec *sqlspec.View, parent *schema.Schema) (*schema.View, error) {
-	v, err := specutil.View(spec, parent, func(c *sqlspec.Column, _ *schema.View) (*schema.Column, error) {
-		return specutil.Column(c, convertColumnType)
-	})
+	v, err := specutil.View(
+		spec, parent,
+		func(c *sqlspec.Column, _ *schema.View) (*schema.Column, error) {
+			return specutil.Column(c, convertColumnType)
+		},
+		func(i *sqlspec.Index, v *schema.View) (*schema.Index, error) {
+			return nil, fmt.Errorf("unexpected view index %s.%s", v.Name, i.Name)
+		},
+	)
 	if err != nil {
 		return nil, err
 	}
@@ -270,7 +278,7 @@ func convertColumnType(spec *sqlspec.Column) (schema.Type, error) {
 
 // schemaSpec converts from a concrete MySQL schema to Atlas specification.
 func schemaSpec(s *schema.Schema) (*specutil.SchemaSpec, error) {
-	spec, err := specutil.FromSchema(s, tableSpec, viewSpec)
+	spec, err := specutil.FromSchema(s, specFuncs)
 	if err != nil {
 		return nil, err
 	}
@@ -318,9 +326,13 @@ func tableSpec(t *schema.Table) (*sqlspec.Table, error) {
 
 // viewSpec converts from a concrete MySQL schema.View to a sqlspec.View.
 func viewSpec(view *schema.View) (*sqlspec.View, error) {
-	spec, err := specutil.FromView(view, func(c *schema.Column, _ *schema.View) (*sqlspec.Column, error) {
-		return specutil.FromColumn(c, columnTypeSpec)
-	})
+	spec, err := specutil.FromView(
+		view,
+		func(c *schema.Column, _ *schema.View) (*sqlspec.Column, error) {
+			return specutil.FromColumn(c, columnTypeSpec)
+		},
+		indexSpec,
+	)
 	if err != nil {
 		return nil, err
 	}
