@@ -5,6 +5,7 @@
 package postgres
 
 import (
+	"ariga.io/atlas/schemahcl"
 	"bytes"
 	"context"
 	"database/sql"
@@ -34,8 +35,7 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 	if opts == nil {
 		opts = &schema.InspectRealmOption{}
 	}
-	r := schema.NewRealm(schemas...).SetCollation(i.collate)
-	r.Attrs = append(r.Attrs, &CType{V: i.ctype})
+	r := schema.NewRealm(schemas...)
 	if len(schemas) > 0 {
 		mode := sqlx.ModeInspectRealm(opts)
 		if mode.Is(schema.InspectTables) {
@@ -49,9 +49,19 @@ func (i *inspect) InspectRealm(ctx context.Context, opts *schema.InspectRealmOpt
 				return nil, err
 			}
 		}
+		if mode.Is(schema.InspectFuncs) {
+			if err := i.inspectFuncs(ctx, r, nil); err != nil {
+				return nil, err
+			}
+		}
 		if err := i.inspectEnums(ctx, r); err != nil {
 			return nil, err
 		}
+		if mode.Is(schema.InspectTypes) {
+			if err := i.inspectTypes(ctx, r, nil); err != nil {
+				return nil, err
+			}
+		}
 	}
 	return sqlx.ExcludeRealm(r, opts.Exclude)
 }
@@ -72,22 +82,32 @@ func (i *inspect) InspectSchema(ctx context.Context, name string, opts *schema.I
 	if opts == nil {
 		opts = &schema.InspectOptions{}
 	}
-	r := schema.NewRealm(schemas...).SetCollation(i.collate)
-	r.Attrs = append(r.Attrs, &CType{V: i.ctype})
-	if sqlx.ModeInspectSchema(opts).Is(schema.InspectTables) {
+	r := schema.NewRealm(schemas...)
+	mode := sqlx.ModeInspectSchema(opts)
+	if mode.Is(schema.InspectTables) {
 		if err := i.inspectTables(ctx, r, opts); err != nil {
 			return nil, err
 		}
 		sqlx.LinkSchemaTables(schemas)
 	}
-	if sqlx.ModeInspectSchema(opts).Is(schema.InspectViews) {
+	if mode.Is(schema.InspectViews) {
 		if err := i.inspectViews(ctx, r, opts); err != nil {
 			return nil, err
 		}
 	}
+	if mode.Is(schema.InspectFuncs) {
+		if err := i.inspectFuncs(ctx, r, opts); err != nil {
+			return nil, err
+		}
+	}
 	if err := i.inspectEnums(ctx, r); err != nil {
 		return nil, err
 	}
+	if mode.Is(schema.InspectTypes) {
+		if err := i.inspectTypes(ctx, r, opts); err != nil {
+			return nil, err
+		}
+	}
 	return sqlx.ExcludeSchema(r.Schemas[0], opts.Exclude)
 }
 
@@ -99,17 +119,7 @@ func (i *inspect) inspectTables(ctx context.Context, r *schema.Realm, opts *sche
 		if len(s.Tables) == 0 {
 			continue
 		}
-		if err := i.columns(ctx, s, queryScope{
-			exec: i.querySchema,
-			append: func(s *schema.Schema, table string, column *schema.Column) error {
-				t, ok := s.Table(table)
-				if !ok {
-					return fmt.Errorf("table %q was not found in schema", table)
-				}
-				t.AddColumns(column)
-				return nil
-			},
-		}); err != nil {
+		if err := i.columns(ctx, s); err != nil {
 			return err
 		}
 		if err := i.indexes(ctx, s); err != nil {
@@ -176,24 +186,19 @@ func (i *inspect) tables(ctx context.Context, realm *schema.Realm, opts *schema.
 	return rows.Close()
 }
 
-type queryScope struct {
-	exec   func(context.Context, string, *schema.Schema) (*sql.Rows, error)
-	append func(*schema.Schema, string, *schema.Column) error
-}
-
 // columns queries and appends the columns of the given table.
-func (i *inspect) columns(ctx context.Context, s *schema.Schema, scope queryScope) error {
+func (i *inspect) columns(ctx context.Context, s *schema.Schema) error {
 	query := columnsQuery
 	if i.crdb {
 		query = crdbColumnsQuery
 	}
-	rows, err := scope.exec(ctx, query, s)
+	rows, err := i.querySchema(ctx, query, s)
 	if err != nil {
 		return fmt.Errorf("postgres: querying schema %q columns: %w", s.Name, err)
 	}
 	defer rows.Close()
 	for rows.Next() {
-		if err := i.addColumn(s, rows, scope); err != nil {
+		if err := i.addColumn(s, rows); err != nil {
 			return fmt.Errorf("postgres: %w", err)
 		}
 	}
@@ -201,7 +206,7 @@ func (i *inspect) columns(ctx context.Context, s *schema.Schema, scope queryScop
 }
 
 // addColumn scans the current row and adds a new column from it to the scope (table or view).
-func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows, scope queryScope) (err error) {
+func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows) (err error) {
 	var (
 		typid, typelem, maxlen, precision, timeprecision, scale, seqstart, seqinc, seqlast                                                  sql.NullInt64
 		table, name, typ, fmtype, nullable, defaults, identity, genidentity, genexpr, charset, collate, comment, typtype, elemtyp, interval sql.NullString
@@ -212,11 +217,21 @@ func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows, scope queryScope)
 	); err != nil {
 		return err
 	}
+	t, ok := s.Table(table.String)
+	if !ok {
+		return fmt.Errorf("table %q was not found in schema", table.String)
+	}
 	c := &schema.Column{
 		Name: name.String,
 		Type: &schema.ColumnType{
-			Raw:  typ.String,
 			Null: nullable.String == "YES",
+			Raw: func() string {
+				// For domains, use the domain type instead of the base type.
+				if typtype.String == "d" {
+					return fmtype.String
+				}
+				return typ.String
+			}(),
 		},
 	}
 	c.Type.Type, err = columnType(&columnDesc{
@@ -233,7 +248,7 @@ func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows, scope queryScope)
 		timePrecision: &timeprecision.Int64,
 	})
 	if defaults.Valid {
-		defaultExpr(c, defaults.String)
+		columnDefault(c, defaults.String)
 	}
 	if identity.String == "YES" {
 		c.Attrs = append(c.Attrs, &Identity{
@@ -259,7 +274,8 @@ func (i *inspect) addColumn(s *schema.Schema, rows *sql.Rows, scope queryScope)
 	if sqlx.ValidString(collate) {
 		c.SetCollation(collate.String)
 	}
-	return scope.append(s, table.String, c)
+	t.AddColumns(c)
+	return nil
 }
 
 // enumValues fills enum columns with their values from the database.
@@ -336,30 +352,66 @@ func (i *inspect) inspectEnums(ctx context.Context, r *schema.Realm) error {
 
 // indexes queries and appends the indexes of the given table.
 func (i *inspect) indexes(ctx context.Context, s *schema.Schema) error {
-	var query string
-	switch {
-	case i.conn.crdb:
+	if i.crdb {
 		return i.crdbIndexes(ctx, s)
-	case i.supportsIndexNullsDistinct():
-		query = indexesAbove15
-	case i.supportsIndexInclude():
-		query = indexesAbove11
-	default:
-		query = indexesBelow11
 	}
-	rows, err := i.querySchema(ctx, query, s)
+	rows, err := i.querySchema(ctx, i.indexesQuery(), s)
 	if err != nil {
 		return fmt.Errorf("postgres: querying schema %q indexes: %w", s.Name, err)
 	}
 	defer rows.Close()
-	if err := i.addIndexes(s, rows); err != nil {
+	if err := i.addIndexes(s, rows, queryScope{
+		hasT: func(tv string) bool {
+			_, ok := s.Table(tv)
+			return ok
+		},
+		setPK: func(tv string, idx *schema.Index) error {
+			if t, ok := s.Table(tv); ok {
+				t.SetPrimaryKey(idx)
+				return nil
+			}
+			return fmt.Errorf("postgres: table %q for primary key was not found in schema", tv)
+		},
+		addIndex: func(tv string, idx *schema.Index) error {
+			if t, ok := s.Table(tv); ok {
+				t.AddIndexes(idx)
+				return nil
+			}
+			return fmt.Errorf("postgres: table %q for index was not found in schema", tv)
+		},
+		column: func(tv, name string) (*schema.Column, bool) {
+			if t, ok := s.Table(tv); ok {
+				return t.Column(name)
+			}
+			return nil, false
+		},
+	}); err != nil {
 		return err
 	}
 	return rows.Err()
 }
 
+func (i *inspect) indexesQuery() (q string) {
+	switch {
+	case i.supportsIndexNullsDistinct():
+		q = indexesAbove15
+	case i.supportsIndexInclude():
+		q = indexesAbove11
+	default:
+		q = indexesBelow11
+	}
+	return
+}
+
+type queryScope struct {
+	hasT     func(tv string) bool
+	setPK    func(tv string, idx *schema.Index) error
+	addIndex func(tv string, idx *schema.Index) error
+	column   func(tv, name string) (*schema.Column, bool)
+}
+
 // addIndexes scans the rows and adds the indexes to the table.
-func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
+func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows, scope queryScope) error {
 	names := make(map[string]*schema.Index)
 	for rows.Next() {
 		var (
@@ -374,8 +426,7 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 		); err != nil {
 			return fmt.Errorf("postgres: scanning indexes for schema %q: %w", s.Name, err)
 		}
-		t, ok := s.Table(table)
-		if !ok {
+		if !scope.hasT(table) {
 			return fmt.Errorf("table %q was not found in schema", table)
 		}
 		idx, ok := names[name]
@@ -383,7 +434,6 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 			idx = &schema.Index{
 				Name:   name,
 				Unique: uniq,
-				Table:  t,
 				Attrs: []schema.Attr{
 					&IndexType{T: typ},
 				},
@@ -414,10 +464,14 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 				idx.AddAttrs(&IndexNullsDistinct{V: false})
 			}
 			names[name] = idx
+			var err error
 			if primary {
-				t.PrimaryKey = idx
+				err = scope.setPK(table, idx)
 			} else {
-				t.Indexes = append(t.Indexes, idx)
+				err = scope.addIndex(table, idx)
+			}
+			if err != nil {
+				return err
 			}
 		}
 		part := &schema.IndexPart{SeqNo: len(idx.Parts) + 1, Desc: desc.Bool}
@@ -429,7 +483,7 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 		}
 		switch {
 		case included:
-			c, ok := t.Column(column.String)
+			c, ok := scope.column(table, column.String)
 			if !ok {
 				return fmt.Errorf("postgres: INCLUDE column %q was not found for index %q", column.String, idx.Name)
 			}
@@ -438,7 +492,7 @@ func (i *inspect) addIndexes(s *schema.Schema, rows *sql.Rows) error {
 			include.Columns = append(include.Columns, c)
 			schema.ReplaceOrAppend(&idx.Attrs, &include)
 		case sqlx.ValidString(column):
-			part.C, ok = t.Column(column.String)
+			part.C, ok = scope.column(table, column.String)
 			if !ok {
 				return fmt.Errorf("postgres: column %q was not found for index %q", column.String, idx.Name)
 			}
@@ -529,7 +583,7 @@ func (i *inspect) fks(ctx context.Context, s *schema.Schema) error {
 		return fmt.Errorf("postgres: querying schema %q foreign keys: %w", s.Name, err)
 	}
 	defer rows.Close()
-	if err := sqlx.SchemaFKs(s, rows); err != nil {
+	if err := sqlx.TypedSchemaFKs[*ReferenceOption](s, rows); err != nil {
 		return fmt.Errorf("postgres: %w", err)
 	}
 	return rows.Err()
@@ -651,7 +705,7 @@ func nArgs(start, n int) string {
 // nextval('<optional (quoted) schema>.<sequence name>'::regclass).
 var reNextval = regexp.MustCompile(`(?i) *nextval\('(?:"?[\w$]+"?\.)?"?([\w$]+_[\w$]+_seq)"?'(?:::regclass)*\) *$`)
 
-func defaultExpr(c *schema.Column, s string) {
+func columnDefault(c *schema.Column, s string) {
 	switch m := reNextval.FindStringSubmatch(s); {
 	// The definition of "<column> <serial type>" is equivalent to specifying:
 	// "<column> <int type> NOT NULL DEFAULT nextval('<table>_<column>_seq')".
@@ -665,26 +719,33 @@ func defaultExpr(c *schema.Column, s string) {
 		st.SetType(tt)
 		c.Type.Raw = st.T
 		c.Type.Type = st
-	case sqlx.IsLiteralBool(s), sqlx.IsLiteralNumber(s), sqlx.IsQuoted(s, '\''):
-		c.Default = &schema.Literal{V: s}
 	default:
-		var x schema.Expr = &schema.RawExpr{X: s}
-		// Try casting or fallback to raw expressions (e.g. column text[] has the default of '{}':text[]).
-		if v, ok := canConvert(c.Type, s); ok {
-			x = &schema.Literal{V: v}
-		}
-		c.Default = x
+		c.Default = defaultExpr(c.Type.Type, s)
 	}
 }
 
-func canConvert(t *schema.ColumnType, x string) (string, bool) {
+func defaultExpr(t schema.Type, s string) schema.Expr {
+	switch {
+	case sqlx.IsLiteralBool(s), sqlx.IsLiteralNumber(s), sqlx.IsQuoted(s, '\''):
+		return &schema.Literal{V: s}
+	default:
+		var x schema.Expr = &schema.RawExpr{X: s}
+		// Try casting or fallback to raw expressions (e.g. column text[] has the default of '{}':text[]).
+		if v, ok := canConvert(t, s); ok {
+			x = &schema.Literal{V: v}
+		}
+		return x
+	}
+}
+
+func canConvert(t schema.Type, x string) (string, bool) {
 	i := strings.LastIndex(x, "::")
 	if i == -1 || !sqlx.IsQuoted(x[:i], '\'') {
 		return "", false
 	}
 	q := x[0:i]
 	x = x[1 : i-1]
-	switch t.Type.(type) {
+	switch t.(type) {
 	case *enumType:
 		return q, true
 	case *schema.BoolType:
@@ -702,18 +763,19 @@ func canConvert(t *schema.ColumnType, x string) (string, bool) {
 }
 
 type (
-	// CType describes the character classification setting (LC_CTYPE).
-	CType struct {
-		schema.Attr
-		V string
-	}
-
 	// UserDefinedType defines a user-defined type attribute.
 	UserDefinedType struct {
 		schema.Type
 		T string
 	}
 
+	// PseudoType defines a non-column pseudo-type, such as function arguments and return types.
+	// https://www.postgresql.org/docs/current/datatype-pseudo.html
+	PseudoType struct {
+		schema.Type
+		T string // e.g., void, any, cstring, etc.
+	}
+
 	// enumType represents an enum type. It serves aa intermediate representation of a Postgres enum type,
 	// to temporary save TypeID and TypeName of an enum column until the enum values can be extracted.
 	enumType struct {
@@ -739,6 +801,18 @@ type (
 		Len int64
 	}
 
+	// DomainType represents a domain type.
+	// https://www.postgresql.org/docs/current/domains.html
+	DomainType struct {
+		schema.Type
+		schema.Object
+		T       string          // Type name.
+		Schema  *schema.Schema  // Optional schema.
+		Null    bool            // Nullability.
+		Default schema.Expr     // Default value.
+		Checks  []*schema.Check // Check constraints.
+	}
+
 	// IntervalType defines an interval type.
 	// https://postgresql.org/docs/current/datatype-datetime.html
 	IntervalType struct {
@@ -934,8 +1008,44 @@ type (
 	Cascade struct {
 		schema.Clause
 	}
+
+	// ReferenceOption describes the ON DELETE and ON UPDATE options for foreign keys.
+	ReferenceOption schema.ReferenceOption
 )
 
+// Ref returns a reference to the domain type.
+func (d *DomainType) Ref() *schemahcl.Ref {
+	return &schemahcl.Ref{V: "$domain." + d.T}
+}
+
+// String implements fmt.Stringer interface.
+func (o ReferenceOption) String() string {
+	return string(o)
+}
+
+// Scan implements sql.Scanner interface.
+func (o *ReferenceOption) Scan(v any) error {
+	var s sql.NullString
+	if err := s.Scan(v); err != nil {
+		return err
+	}
+	switch strings.ToLower(s.String) {
+	case "a":
+		*o = ReferenceOption(schema.NoAction)
+	case "r":
+		*o = ReferenceOption(schema.Restrict)
+	case "c":
+		*o = ReferenceOption(schema.Cascade)
+	case "n":
+		*o = ReferenceOption(schema.SetNull)
+	case "d":
+		*o = ReferenceOption(schema.SetDefault)
+	default:
+		return fmt.Errorf("unknown reference option: %q", s.String)
+	}
+	return nil
+}
+
 // IsUnique reports if the type is unique constraint.
 func (c Constraint) IsUnique() bool { return strings.ToLower(c.T) == "u" }
 
@@ -1104,33 +1214,38 @@ func newIndexStorage(opts string) (*IndexStorageParams, error) {
 	return params, nil
 }
 
-// reEnumType extracts the enum type and an option schema qualifier.
-var reEnumType = regexp.MustCompile(`^(?:(".+"|\w+)\.)?(".+"|\w+)$`)
+// reFmtType extracts the formatted type and an option schema qualifier.
+var reFmtType = regexp.MustCompile(`^(?:(".+"|\w+)\.)?(".+"|\w+)$`)
 
-func newEnumType(t string, id int64) *enumType {
-	var (
-		e     = &enumType{T: t, ID: id}
-		parts = reEnumType.FindStringSubmatch(e.T)
-		r     = func(s string) string {
-			s = strings.ReplaceAll(s, `""`, `"`)
-			if len(s) > 1 && s[0] == '"' && s[len(s)-1] == '"' {
-				s = s[1 : len(s)-1]
-			}
-			return s
+// parseFmtType parses the formatted type returned from pg_catalog.format_type
+// and extract the schema and type name.
+func parseFmtType(t string) (s, n string) {
+	n = t
+	parts := reFmtType.FindStringSubmatch(t)
+	r := func(s string) string {
+		s = strings.ReplaceAll(s, `""`, `"`)
+		if len(s) > 1 && s[0] == '"' && s[len(s)-1] == '"' {
+			s = s[1 : len(s)-1]
 		}
-	)
+		return s
+	}
 	if len(parts) > 1 {
-		e.Schema = r(parts[1])
+		s = r(parts[1])
 	}
 	if len(parts) > 2 {
-		e.T = r(parts[2])
+		n = r(parts[2])
 	}
-	return e
+	return s, n
+}
+
+func newEnumType(t string, id int64) *enumType {
+	s, n := parseFmtType(t)
+	return &enumType{T: n, Schema: s, ID: id}
 }
 
 const (
 	// Query to list runtime parameters.
-	paramsQuery = `SELECT setting FROM pg_settings WHERE name IN ('lc_collate', 'lc_ctype', 'server_version_num', 'crdb_version') ORDER BY name DESC`
+	paramsQuery = `SELECT setting FROM pg_settings WHERE name IN ('server_version_num', 'crdb_version') ORDER BY name DESC`
 
 	// Query to list database schemas.
 	schemasQuery = `
@@ -1267,8 +1382,8 @@ SELECT
     fk.referenced_table_name,
     a2.attname AS referenced_column_name,
     fk.referenced_schema_name,
-    rc.update_rule,
-    rc.delete_rule
+    fk.confupdtype,
+    fk.confdeltype
 	FROM 
 	    (
 	    	SELECT
@@ -1281,7 +1396,9 @@ SELECT
 	      		ns2.nspname AS referenced_schema_name,
 	      		generate_series(1,array_length(con.conkey,1)) as ord,
 	      		unnest(con.conkey) AS conkey,
-	      		unnest(con.confkey) AS confkey
+	      		unnest(con.confkey) AS confkey,
+	      		con.confupdtype,
+	      		con.confdeltype
 	    	FROM pg_constraint con
 	    	JOIN pg_class t1 ON t1.oid = con.conrelid
 	    	JOIN pg_class t2 ON t2.oid = con.confrelid
@@ -1293,7 +1410,6 @@ SELECT
 	) AS fk
 	JOIN pg_attribute a1 ON a1.attnum = fk.conkey AND a1.attrelid = fk.conrelid
 	JOIN pg_attribute a2 ON a2.attnum = fk.confkey AND a2.attrelid = fk.confrelid
-	JOIN information_schema.referential_constraints rc ON rc.constraint_name = fk.constraint_name AND rc.constraint_schema = fk.schema_name
 	ORDER BY
 	    fk.conrelid, fk.constraint_name, fk.ord
 `
