@@ -145,7 +145,7 @@ func TestDriver_InspectTable(t *testing.T) {
 					{Name: "c32", Type: &schema.ColumnType{Raw: "bigserial", Type: &SerialType{T: "bigserial", SequenceName: "t1_c32_seq"}}},
 					{Name: "c33", Type: &schema.ColumnType{Raw: `status".`, Type: statusE}, Default: &schema.Literal{V: "'unknown'"}},
 					{Name: "c34", Type: &schema.ColumnType{Raw: "ARRAY", Type: &ArrayType{T: "state[]", Type: stateE}}},
-					{Name: "c35", Type: &schema.ColumnType{Raw: "character", Type: &UserDefinedType{T: "domain_char"}}},
+					{Name: "c35", Type: &schema.ColumnType{Raw: "domain_char", Type: &UserDefinedType{T: "domain_char"}}},
 					{Name: "c36", Type: &schema.ColumnType{Raw: "tsvector", Type: &TextSearchType{T: "tsvector"}}},
 					{Name: "c37", Type: &schema.ColumnType{Raw: "tsquery", Type: &TextSearchType{T: "tsquery"}}},
 					{Name: "c38", Type: &schema.ColumnType{Raw: "datemultirange", Type: &RangeType{T: "datemultirange"}}},
@@ -247,13 +247,13 @@ users      | uid         | integer             | int       | NO          |
 				m.ExpectQuery(queryFKs).
 					WithArgs("public", "users").
 					WillReturnRows(sqltest.Rows(`
-constraint_name | table_name | column_name | table_schema | referenced_table_name | referenced_column_name | referenced_schema_name | update_rule | delete_rule
+constraint_name | table_name | column_name | table_schema | referenced_table_name | referenced_column_name | referenced_schema_name | confupdtype | condeltype
 -----------------+------------+-------------+--------------+-----------------------+------------------------+------------------------+-------------+-------------
-multi_column    | users      | id          | public       | t1                    | gid                    | public                 | NO ACTION   | CASCADE
-multi_column    | users      | id          | public       | t1                    | xid                    | public                 | NO ACTION   | CASCADE
-multi_column    | users      | oid         | public       | t1                    | gid                    | public                 | NO ACTION   | CASCADE
-multi_column    | users      | oid         | public       | t1                    | xid                    | public                 | NO ACTION   | CASCADE
-self_reference  | users      | uid         | public       | users                 | id                     | public                 | NO ACTION   | CASCADE
+multi_column    | users      | id          | public       | t1                    | gid                    | public                 | a            | c
+multi_column    | users      | id          | public       | t1                    | xid                    | public                 | a            | c
+multi_column    | users      | oid         | public       | t1                    | gid                    | public                 | a            | c
+multi_column    | users      | oid         | public       | t1                    | xid                    | public                 | a            | c
+self_reference  | users      | uid         | public       | users                 | id                     | public                 | a            | c
 `))
 				m.noChecks()
 				m.noEnums()
@@ -396,7 +396,7 @@ logs3      | c5         | integer   | integer   | NO          |                |
 		WillReturnRows(sqlmock.NewRows([]string{"table_name", "constraint_name", "expression", "column_name", "column_indexes"}))
 	mk.noEnums()
 	s, err := drv.InspectSchema(context.Background(), "", &schema.InspectOptions{
-		Mode: ^schema.InspectViews,
+		Mode: schema.InspectSchemas | schema.InspectTables,
 	})
 	require.NoError(t, err)
 
@@ -431,12 +431,10 @@ func TestDriver_InspectCRDBSchema(t *testing.T) {
 	mk := mock{m}
 	mk.ExpectQuery(sqltest.Escape(paramsQuery)).
 		WillReturnRows(sqltest.Rows(`
-					setting
-				------------
-				130000
-				en_US.utf8
-				en_US.utf8
-				cockroach
+	setting
+------------
+130000
+cockroach
 				`))
 	drv, err := Open(db)
 	require.NoError(t, err)
@@ -475,7 +473,7 @@ users       | idx5       | c           | false   | false  |                 | CR
 	mk.noChecks()
 	mk.noEnums()
 	s, err := drv.InspectSchema(context.Background(), "public", &schema.InspectOptions{
-		Mode: ^schema.InspectViews,
+		Mode: schema.InspectSchemas | schema.InspectTables,
 	})
 	require.NoError(t, err)
 	tbl := s.Tables[0]
@@ -519,7 +517,7 @@ func TestDriver_InspectSchema(t *testing.T) {
 		WillReturnRows(sqlmock.NewRows([]string{"table_schema", "table_name", "comment", "partition_attrs", "partition_strategy", "partition_exprs"}))
 	mk.noEnums()
 	s, err := drv.InspectSchema(context.Background(), "", &schema.InspectOptions{
-		Mode: ^schema.InspectViews,
+		Mode: schema.InspectSchemas | schema.InspectTables,
 	})
 	require.NoError(t, err)
 	require.EqualValues(t, func() *schema.Schema {
@@ -527,15 +525,6 @@ func TestDriver_InspectSchema(t *testing.T) {
 			Schemas: []*schema.Schema{
 				schema.New("test").SetComment("boring"),
 			},
-			// Server default configuration.
-			Attrs: []schema.Attr{
-				&schema.Collation{
-					V: "en_US.utf8",
-				},
-				&CType{
-					V: "en_US.utf8",
-				},
-			},
 		}
 		r.Schemas[0].Realm = r
 		return r.Schemas[0]
@@ -562,7 +551,7 @@ func TestDriver_Realm(t *testing.T) {
 	m.ExpectQuery(sqltest.Escape(fmt.Sprintf(enumsQuery, "$1, $2"))).
 		WillReturnRows(sqlmock.NewRows([]string{"schema_name", "enum_name", "comment", "enum_type", "enum_value"}))
 	realm, err := drv.InspectRealm(context.Background(), &schema.InspectRealmOption{
-		Mode: ^schema.InspectViews,
+		Mode: schema.InspectSchemas | schema.InspectTables,
 	})
 	require.NoError(t, err)
 	require.EqualValues(t, func() *schema.Realm {
@@ -575,15 +564,6 @@ func TestDriver_Realm(t *testing.T) {
 					Name: "public",
 				},
 			},
-			// Server default configuration.
-			Attrs: []schema.Attr{
-				&schema.Collation{
-					V: "en_US.utf8",
-				},
-				&CType{
-					V: "en_US.utf8",
-				},
-			},
 		}
 		r.Schemas[0].Realm = r
 		r.Schemas[1].Realm = r
@@ -605,7 +585,7 @@ func TestDriver_Realm(t *testing.T) {
 		WillReturnRows(sqlmock.NewRows([]string{"schema_name", "enum_name", "comment", "enum_type", "enum_value"}))
 	realm, err = drv.InspectRealm(context.Background(), &schema.InspectRealmOption{
 		Schemas: []string{"test", "public"},
-		Mode:    ^schema.InspectViews,
+		Mode:    schema.InspectSchemas | schema.InspectTables,
 	})
 	require.NoError(t, err)
 	require.EqualValues(t, func() *schema.Realm {
@@ -618,15 +598,6 @@ func TestDriver_Realm(t *testing.T) {
 					Name: "public",
 				},
 			},
-			// Server default configuration.
-			Attrs: []schema.Attr{
-				&schema.Collation{
-					V: "en_US.utf8",
-				},
-				&CType{
-					V: "en_US.utf8",
-				},
-			},
 		}
 		r.Schemas[0].Realm = r
 		r.Schemas[1].Realm = r
@@ -656,15 +627,6 @@ func TestDriver_Realm(t *testing.T) {
 					Name: "test",
 				},
 			},
-			// Server default configuration.
-			Attrs: []schema.Attr{
-				&schema.Collation{
-					V: "en_US.utf8",
-				},
-				&CType{
-					V: "en_US.utf8",
-				},
-			},
 		}
 		r.Schemas[0].Realm = r
 		return r
@@ -698,15 +660,6 @@ func TestInspectMode_InspectRealm(t *testing.T) {
 					Name: "public",
 				},
 			},
-			// Server default configuration.
-			Attrs: []schema.Attr{
-				&schema.Collation{
-					V: "en_US.utf8",
-				},
-				&CType{
-					V: "en_US.utf8",
-				},
-			},
 		}
 		r.Schemas[0].Realm = r
 		r.Schemas[1].Realm = r
@@ -737,8 +690,6 @@ func (m mock) version(version string) {
   setting
 ------------
  ` + version + `
- en_US.utf8
- en_US.utf8
 `))
 }
 
