@@ -26,7 +26,7 @@ var DefaultPlan migrate.PlanApplier = &planApply{conn: &conn{ExecQuerier: sqlx.N
 type planApply struct{ *conn }
 
 // PlanChanges returns a migration plan for the given schema changes.
-func (p *planApply) PlanChanges(_ context.Context, name string, changes []schema.Change, opts ...migrate.PlanOption) (*migrate.Plan, error) {
+func (p *planApply) PlanChanges(ctx context.Context, name string, changes []schema.Change, opts ...migrate.PlanOption) (*migrate.Plan, error) {
 	s := &state{
 		conn: p.conn,
 		Plan: migrate.Plan{
@@ -37,6 +37,9 @@ func (p *planApply) PlanChanges(_ context.Context, name string, changes []schema
 	for _, o := range opts {
 		o(&s.PlanOptions)
 	}
+	if err := verifyChanges(ctx, changes); err != nil {
+		return nil, err
+	}
 	if err := s.plan(changes); err != nil {
 		return nil, err
 	}
@@ -80,8 +83,7 @@ func (s *state) plan(changes []schema.Change) error {
 	}
 	var (
 		views []schema.Change
-		dropT []*schema.DropTable
-		dropO []*schema.DropObject
+		drop  struct{ T, O, F []schema.Change }
 	)
 	for _, c := range planned {
 		switch c := c.(type) {
@@ -91,12 +93,26 @@ func (s *state) plan(changes []schema.Change) error {
 			err = s.modifyTable(c)
 		case *schema.RenameTable:
 			s.renameTable(c)
+		case *schema.AddFunc:
+			err = s.addFunc(c)
+		case *schema.AddProc:
+			err = s.addProc(c)
+		case *schema.ModifyFunc:
+			err = s.modifyFunc(c)
+		case *schema.ModifyProc:
+			err = s.modifyProc(c)
+		case *schema.RenameFunc:
+			err = s.renameFunc(c)
+		case *schema.RenameProc:
+			err = s.renameProc(c)
 		case *schema.AddView, *schema.DropView, *schema.ModifyView, *schema.RenameView:
 			views = append(views, c)
 		case *schema.DropTable:
-			dropT = append(dropT, c)
+			drop.T = append(drop.T, c)
 		case *schema.DropObject:
-			dropO = append(dropO, c)
+			drop.O = append(drop.O, c)
+		case *schema.DropFunc, *schema.DropProc:
+			drop.F = append(drop.F, c)
 		default:
 			err = fmt.Errorf("unsupported change %T", c)
 		}
@@ -118,24 +134,25 @@ func (s *state) plan(changes []schema.Change) error {
 		case *schema.RenameView:
 			s.renameView(c)
 		}
-	}
-	for _, c := range dropT {
-		if err := s.dropTable(c); err != nil {
+		if err != nil {
 			return err
 		}
 	}
-	for _, c := range dropO {
-		e, ok := c.O.(*schema.EnumType)
-		if !ok {
-			return fmt.Errorf("unsupported object %T", c.O)
+	for _, c := range append(drop.T, append(drop.O, drop.F...)...) {
+		var err error
+		switch c := c.(type) {
+		case *schema.DropTable:
+			err = s.dropTable(c)
+		case *schema.DropObject:
+			err = s.dropObject(c)
+		case *schema.DropFunc:
+			err = s.dropFunc(c)
+		case *schema.DropProc:
+			err = s.dropProc(c)
+		}
+		if err != nil {
+			return err
 		}
-		create, drop := s.createDropEnum(e)
-		s.append(&migrate.Change{
-			Source:  c,
-			Cmd:     drop,
-			Reverse: create,
-			Comment: fmt.Sprintf("drop enum type %q", e.T),
-		})
 	}
 	return nil
 }
@@ -198,19 +215,11 @@ func (s *state) topLevel(changes []schema.Change) ([]schema.Change, error) {
 				Comment: fmt.Sprintf("Drop schema named %q", c.S.Name),
 			})
 		case *schema.AddObject:
-			e, ok := c.O.(*schema.EnumType)
-			if !ok {
-				return nil, fmt.Errorf("unsupported object %T", c.O)
+			if err := s.addObject(c); err != nil {
+				return nil, err
 			}
-			create, drop := s.createDropEnum(e)
-			s.append(&migrate.Change{
-				Source:  c,
-				Cmd:     create,
-				Reverse: drop,
-				Comment: fmt.Sprintf("create enum type %q", e.T),
-			})
 		case *schema.ModifyObject:
-			if err := s.alterEnum(c); err != nil {
+			if err := s.modifyObject(c); err != nil {
 				return nil, err
 			}
 		case *schema.RenameObject:
@@ -874,7 +883,7 @@ func (s *state) addIndexes(t *schema.Table, adds ...*schema.AddIndex) error {
 }
 
 func (s *state) column(b *sqlx.Builder, c *schema.Column) error {
-	f, err := s.formatType(c)
+	f, err := s.formatType(c.Type.Type)
 	if err != nil {
 		return err
 	}
@@ -923,10 +932,17 @@ func (s *state) column(b *sqlx.Builder, c *schema.Column) error {
 
 // columnDefault writes the default value of column to the builder.
 func (s *state) columnDefault(b *sqlx.Builder, c *schema.Column) {
-	switch x := c.Default.(type) {
+	if c.Default != nil {
+		s.formatDefault(b, c.Type.Type, c.Default)
+	}
+}
+
+// formatDefault writes the default value of column to the builder.
+func (s *state) formatDefault(b *sqlx.Builder, t schema.Type, x schema.Expr) {
+	switch x := x.(type) {
 	case *schema.Literal:
 		v := x.V
-		switch c.Type.Type.(type) {
+		switch t.(type) {
 		case *schema.BoolType, *schema.DecimalType, *schema.IntegerType, *schema.FloatType:
 		default:
 			v = quote(v)
@@ -934,7 +950,7 @@ func (s *state) columnDefault(b *sqlx.Builder, c *schema.Column) {
 		b.P("DEFAULT", v)
 	case *schema.RawExpr:
 		// Ignore identity functions added by the Differ.
-		if _, ok := c.Type.Type.(*SerialType); !ok {
+		if _, ok := t.(*SerialType); !ok {
 			b.P("DEFAULT", x.X)
 		}
 	}
@@ -1205,18 +1221,24 @@ func (s *state) createDropEnum(e *schema.EnumType) (string, string) {
 }
 
 func (s *state) enumIdent(e *schema.EnumType) string {
+	return s.typeIdent(e.Schema, e.T)
+}
+
+func (s *state) domainIdent(d *DomainType) string {
+	return s.typeIdent(d.Schema, d.T)
+}
+
+func (s *state) typeIdent(ns *schema.Schema, name string) string {
 	switch {
 	// In case the plan uses a specific schema qualifier.
 	case s.SchemaQualifier != nil:
 		if *s.SchemaQualifier != "" {
-			return fmt.Sprintf("%q.%q", *s.SchemaQualifier, e.T)
+			return fmt.Sprintf("%q.%q", *s.SchemaQualifier, name)
 		}
-		return strconv.Quote(e.T)
-	case e.Schema != nil:
-		return fmt.Sprintf("%q.%q", e.Schema.Name, e.T)
-	default:
-		return strconv.Quote(e.T)
+	case ns != nil && ns.Name != "":
+		return fmt.Sprintf("%q.%q", ns.Name, name)
 	}
+	return strconv.Quote(name)
 }
 
 // schemaPrefix returns the schema prefix based on the planner config.
@@ -1234,16 +1256,21 @@ func (s *state) schemaPrefix(ns *schema.Schema) string {
 }
 
 // formatType formats the type but takes into account the qualifier.
-func (s *state) formatType(c *schema.Column) (string, error) {
-	switch tt := c.Type.Type.(type) {
+func (s *state) formatType(t schema.Type) (string, error) {
+	switch t := t.(type) {
 	case *schema.EnumType:
-		return s.enumIdent(tt), nil
+		return s.enumIdent(t), nil
+	case *DomainType:
+		return s.domainIdent(t), nil
 	case *ArrayType:
-		if e, ok := tt.Type.(*schema.EnumType); ok {
-			return s.enumIdent(e) + "[]", nil
+		switch t := t.Type.(type) {
+		case *schema.EnumType:
+			return s.enumIdent(t) + "[]", nil
+		case *DomainType:
+			return s.domainIdent(t) + "[]", nil
 		}
 	}
-	return FormatType(c.Type.Type)
+	return FormatType(t)
 }
 
 func pkName(t *schema.Table, pk *schema.Index) string {
