@@ -23,12 +23,17 @@ import (
 
 type (
 	doc struct {
-		Tables  []*sqlspec.Table  `spec:"table"`
-		Views   []*sqlspec.View   `spec:"view"`
-		Enums   []*Enum           `spec:"enum"`
-		Schemas []*sqlspec.Schema `spec:"schema"`
+		Tables       []*sqlspec.Table  `spec:"table"`
+		Views        []*sqlspec.View   `spec:"view"`
+		Materialized []*sqlspec.View   `spec:"materialized"`
+		Enums        []*Enum           `spec:"enum"`
+		Domains      []*Domain         `spec:"domain"`
+		Funcs        []*sqlspec.Func   `spec:"function"`
+		Procs        []*sqlspec.Func   `spec:"procedure"`
+		Schemas      []*sqlspec.Schema `spec:"schema"`
 	}
-	// Enum holds a specification for an enum, that can be referenced as a column type.
+
+	// Enum holds a specification for an enum type.
 	Enum struct {
 		Name      string         `spec:",name"`
 		Qualifier string         `spec:",qualifier"`
@@ -36,8 +41,32 @@ type (
 		Values    []string       `spec:"values"`
 		schemahcl.DefaultExtension
 	}
+
+	// Domain holds a specification for a domain type.
+	Domain struct {
+		Name      string           `spec:",name"`
+		Qualifier string           `spec:",qualifier"`
+		Schema    *schemahcl.Ref   `spec:"schema"`
+		Type      *schemahcl.Type  `spec:"type"`
+		Null      bool             `spec:"null"`
+		Default   cty.Value        `spec:"default"`
+		Checks    []*sqlspec.Check `spec:"check"`
+		schemahcl.DefaultExtension
+	}
 )
 
+// merge merges the doc d1 into d.
+func (d *doc) merge(d1 *doc) {
+	d.Tables = append(d.Tables, d1.Tables...)
+	d.Views = append(d.Views, d1.Views...)
+	d.Materialized = append(d.Materialized, d1.Materialized...)
+	d.Domains = append(d.Domains, d1.Domains...)
+	d.Enums = append(d.Enums, d1.Enums...)
+	d.Funcs = append(d.Funcs, d1.Funcs...)
+	d.Procs = append(d.Procs, d1.Procs...)
+	d.Schemas = append(d.Schemas, d1.Schemas...)
+}
+
 // Label returns the defaults label used for the enum resource.
 func (e *Enum) Label() string { return e.Name }
 
@@ -50,8 +79,21 @@ func (e *Enum) SetQualifier(q string) { e.Qualifier = q }
 // SchemaRef returns the schema reference for the enum.
 func (e *Enum) SchemaRef() *schemahcl.Ref { return e.Schema }
 
+// Label returns the defaults label used for the enum resource.
+func (d *Domain) Label() string { return d.Name }
+
+// QualifierLabel returns the qualifier label used for the enum resource, if any.
+func (d *Domain) QualifierLabel() string { return d.Qualifier }
+
+// SetQualifier sets the qualifier label used for the enum resource.
+func (d *Domain) SetQualifier(q string) { d.Qualifier = q }
+
+// SchemaRef returns the schema reference for the enum.
+func (d *Domain) SchemaRef() *schemahcl.Ref { return d.Schema }
+
 func init() {
 	schemahcl.Register("enum", &Enum{})
+	schemahcl.Register("domain", &Domain{})
 }
 
 // evalSpec evaluates an Atlas DDL document into v using the input.
@@ -63,15 +105,16 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 			return err
 		}
 		if err := specutil.Scan(v,
-			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views},
-			&specutil.ScanFuncs{Table: convertTable, View: convertView},
+			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views, Materialized: d.Materialized, Funcs: d.Funcs, Procs: d.Procs},
+			scanFuncs,
 		); err != nil {
 			return fmt.Errorf("specutil: failed converting to *schema.Realm: %w", err)
 		}
-		if len(d.Enums) > 0 {
-			if err := convertEnums(d.Tables, d.Enums, v); err != nil {
-				return err
-			}
+		if err := convertEnums(d.Tables, d.Enums, v); err != nil {
+			return err
+		}
+		if err := convertDomains(d.Tables, d.Domains, v); err != nil {
+			return err
 		}
 	case *schema.Schema:
 		var d doc
@@ -83,19 +126,22 @@ func evalSpec(p *hclparse.Parser, v any, input map[string]cty.Value) error {
 		}
 		r := &schema.Realm{}
 		if err := specutil.Scan(r,
-			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views},
-			&specutil.ScanFuncs{Table: convertTable, View: convertView},
+			&specutil.ScanDoc{Schemas: d.Schemas, Tables: d.Tables, Views: d.Views, Materialized: d.Materialized, Funcs: d.Funcs, Procs: d.Procs},
+			scanFuncs,
 		); err != nil {
 			return err
 		}
 		if err := convertEnums(d.Tables, d.Enums, r); err != nil {
 			return err
 		}
+		if err := convertDomains(d.Tables, d.Domains, r); err != nil {
+			return err
+		}
 		*v = *r.Schemas[0]
 	case schema.Schema, schema.Realm:
 		return fmt.Errorf("postgres: Eval expects a pointer: received %[1]T, expected *%[1]T", v)
 	default:
-		return hclState.Eval(p, v, input)
+		return fmt.Errorf("postgres: unexpected type %T", v)
 	}
 	return nil
 }
@@ -105,25 +151,18 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 	var d doc
 	switch s := v.(type) {
 	case *schema.Schema:
-		var err error
-		doc, err := schemaSpec(s)
+		d1, err := schemaSpec(s)
 		if err != nil {
 			return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 		}
-		d.Tables = doc.Tables
-		d.Views = doc.Views
-		d.Schemas = doc.Schemas
-		d.Enums = doc.Enums
+		d.merge(d1)
 	case *schema.Realm:
 		for _, s := range s.Schemas {
-			doc, err := schemaSpec(s)
+			d1, err := schemaSpec(s)
 			if err != nil {
 				return nil, fmt.Errorf("specutil: failed converting schema to spec: %w", err)
 			}
-			d.Tables = append(d.Tables, doc.Tables...)
-			d.Views = append(d.Views, doc.Views...)
-			d.Schemas = append(d.Schemas, doc.Schemas...)
-			d.Enums = append(d.Enums, doc.Enums...)
+			d.merge(d1)
 		}
 		if err := specutil.QualifyObjects(d.Tables); err != nil {
 			return nil, err
@@ -131,9 +170,21 @@ func MarshalSpec(v any, marshaler schemahcl.Marshaler) ([]byte, error) {
 		if err := specutil.QualifyObjects(d.Views); err != nil {
 			return nil, err
 		}
+		if err := specutil.QualifyObjects(d.Materialized); err != nil {
+			return nil, err
+		}
 		if err := specutil.QualifyObjects(d.Enums); err != nil {
 			return nil, err
 		}
+		if err := specutil.QualifyObjects(d.Domains); err != nil {
+			return nil, err
+		}
+		if err := specutil.QualifyObjects(d.Funcs); err != nil {
+			return nil, err
+		}
+		if err := specutil.QualifyObjects(d.Procs); err != nil {
+			return nil, err
+		}
 		if err := specutil.QualifyReferences(d.Tables, s); err != nil {
 			return nil, err
 		}
@@ -147,6 +198,7 @@ var (
 	hclState = schemahcl.New(append(specOptions,
 		schemahcl.WithTypes("table.column.type", TypeRegistry.Specs()),
 		schemahcl.WithTypes("view.column.type", TypeRegistry.Specs()),
+		schemahcl.WithTypes("materialized.column.type", TypeRegistry.Specs()),
 		schemahcl.WithScopedEnums("view.check_option", schema.ViewCheckOptionLocal, schema.ViewCheckOptionCascaded),
 		schemahcl.WithScopedEnums("table.index.type", IndexTypeBTree, IndexTypeBRIN, IndexTypeHash, IndexTypeGIN, IndexTypeGiST, "GiST", IndexTypeSPGiST, "SPGiST"),
 		schemahcl.WithScopedEnums("table.partition.type", PartitionTypeRange, PartitionTypeList, PartitionTypeHash),
@@ -189,9 +241,20 @@ func convertTable(spec *sqlspec.Table, parent *schema.Schema) (*schema.Table, er
 
 // convertView converts a sqlspec.View to a schema.View.
 func convertView(spec *sqlspec.View, parent *schema.Schema) (*schema.View, error) {
-	v, err := specutil.View(spec, parent, func(c *sqlspec.Column, _ *schema.View) (*schema.Column, error) {
-		return specutil.Column(c, convertColumnType)
-	})
+	v, err := specutil.View(
+		spec, parent,
+		func(c *sqlspec.Column, _ *schema.View) (*schema.Column, error) {
+			return specutil.Column(c, convertColumnType)
+		},
+		func(i *sqlspec.Index, v *schema.View) (*schema.Index, error) {
+			idx, err := convertIndex(i, v.AsTable())
+			if err != nil {
+				return nil, err
+			}
+			idx.Table, idx.View = nil, v
+			return idx, nil
+		},
+	)
 	if err != nil {
 		return nil, err
 	}
@@ -472,6 +535,9 @@ func convertColumnType(spec *sqlspec.Column) (schema.Type, error) {
 // convertEnums converts possibly referenced column types (like enums) to
 // an actual schema.Type and sets it on the correct schema.Column.
 func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error {
+	if len(enums) == 0 {
+		return nil
+	}
 	byName := make(map[string]*schema.EnumType)
 	for _, e := range enums {
 		if byName[e.Name] != nil {
@@ -486,14 +552,14 @@ func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error
 			return fmt.Errorf("schema %q defined on enum %q was not found in realm", ns, e.Name)
 		}
 		e1 := &schema.EnumType{T: e.Name, Schema: es, Values: e.Values}
-		es.Objects = append(es.Objects, e1)
+		es.AddObjects(e1)
 		byName[e.Name] = e1
 	}
 	for _, t := range tables {
 		for _, c := range t.Columns {
 			var enum *schema.EnumType
 			switch {
-			case c.Type.IsRef:
+			case c.Type.IsRefTo("enum"):
 				n, err := enumName(c.Type)
 				if err != nil {
 					return err
@@ -504,11 +570,12 @@ func convertEnums(tables []*sqlspec.Table, enums []*Enum, r *schema.Realm) error
 				}
 				enum = e
 			default:
-				n, ok := arrayType(c.Type.T)
-				if !ok || byName[n] == nil {
-					continue
+				if n, ok := arrayType(c.Type.T); ok {
+					enum = byName[n]
 				}
-				enum = byName[n]
+			}
+			if enum == nil {
+				continue
 			}
 			schemaT, err := specutil.SchemaName(t.Schema)
 			if err != nil {
@@ -555,24 +622,22 @@ func enumRef(n string) *schemahcl.Ref {
 
 // schemaSpec converts from a concrete Postgres schema to Atlas specification.
 func schemaSpec(s *schema.Schema) (*doc, error) {
-	spec, err := specutil.FromSchema(s, tableSpec, viewSpec)
+	spec, err := specutil.FromSchema(s, specFuncs)
 	if err != nil {
 		return nil, err
 	}
 	d := &doc{
-		Tables:  spec.Tables,
-		Views:   spec.Views,
-		Schemas: []*sqlspec.Schema{spec.Schema},
-		Enums:   make([]*Enum, 0, len(s.Objects)),
+		Tables:       spec.Tables,
+		Views:        spec.Views,
+		Materialized: spec.Materialized,
+		Funcs:        spec.Funcs,
+		Procs:        spec.Procs,
+		Schemas:      []*sqlspec.Schema{spec.Schema},
+		Enums:        make([]*Enum, 0, len(s.Objects)),
+		Domains:      make([]*Domain, 0, len(s.Objects)),
 	}
-	for _, o := range s.Objects {
-		if e, ok := o.(*schema.EnumType); ok {
-			d.Enums = append(d.Enums, &Enum{
-				Name:   e.T,
-				Values: e.Values,
-				Schema: specutil.SchemaRef(spec.Schema.Name),
-			})
-		}
+	if err := objectSpec(d, spec, s); err != nil {
+		return nil, err
 	}
 	return d, nil
 }
@@ -598,9 +663,13 @@ func tableSpec(table *schema.Table) (*sqlspec.Table, error) {
 
 // viewSpec converts from a concrete PostgreSQL schema.View to a sqlspec.View.
 func viewSpec(view *schema.View) (*sqlspec.View, error) {
-	spec, err := specutil.FromView(view, func(c *schema.Column, _ *schema.View) (*sqlspec.Column, error) {
-		return specutil.FromColumn(c, columnTypeSpec)
-	})
+	spec, err := specutil.FromView(
+		view,
+		func(c *schema.Column, _ *schema.View) (*sqlspec.Column, error) {
+			return specutil.FromColumn(c, columnTypeSpec)
+		},
+		indexSpec,
+	)
 	if err != nil {
 		return nil, err
 	}
@@ -703,17 +772,24 @@ func fromIdentity(i *Identity) *schemahcl.Resource {
 // columnTypeSpec converts from a concrete Postgres schema.Type into sqlspec.Column Type.
 func columnTypeSpec(t schema.Type) (*sqlspec.Column, error) {
 	// Handle postgres enum types. They cannot be put into the TypeRegistry since their name is dynamic.
-	if e, ok := t.(*schema.EnumType); ok {
+	switch o := t.(type) {
+	case *schema.EnumType:
 		return &sqlspec.Column{Type: &schemahcl.Type{
-			T:     enumRef(e.T).V,
+			T:     enumRef(o.T).V,
 			IsRef: true,
 		}}, nil
+	case *DomainType:
+		return &sqlspec.Column{Type: &schemahcl.Type{
+			T:     o.Ref().V,
+			IsRef: true,
+		}}, nil
+	default:
+		st, err := TypeRegistry.Convert(t)
+		if err != nil {
+			return nil, err
+		}
+		return &sqlspec.Column{Type: st}, nil
 	}
-	st, err := TypeRegistry.Convert(t)
-	if err != nil {
-		return nil, err
-	}
-	return &sqlspec.Column{Type: st}, nil
 }
 
 // TypeRegistry contains the supported TypeSpecs for the Postgres driver.
@@ -789,11 +865,13 @@ var TypeRegistry = schemahcl.NewRegistry(
 		schemahcl.NewTypeSpec("hstore"),
 		schemahcl.NewTypeSpec("sql", schemahcl.WithAttributes(&schemahcl.TypeAttr{Name: "def", Required: true, Kind: reflect.String})),
 	),
-	// PostgreSQL internal and special types.
+	// PostgreSQL internal, pseudo, and special types.
 	schemahcl.WithSpecs(func() (specs []*schemahcl.TypeSpec) {
 		for _, t := range []string{
 			typeOID, typeRegClass, typeRegCollation, typeRegConfig, typeRegDictionary, typeRegNamespace,
 			typeName, typeRegOper, typeRegOperator, typeRegProc, typeRegProcedure, typeRegRole, typeRegType,
+			typeAny, typeAnyElement, typeAnyArray, typeAnyNonArray, typeAnyEnum, typeInternal, typeRecord,
+			typeTrigger, typeVoid, typeUnknown,
 		} {
 			specs = append(specs, schemahcl.NewTypeSpec(t))
 		}
