@@ -85,6 +85,24 @@ func (s *Schema) AddObjects(objs ...Object) *Schema {
 	return s
 }
 
+// AddFuncs appends the given functions to the schema.
+func (s *Schema) AddFuncs(funcs ...*Func) *Schema {
+	for _, f := range funcs {
+		f.Schema = s
+	}
+	s.Funcs = append(s.Funcs, funcs...)
+	return s
+}
+
+// AddProcs appends the given procedures to the schema.
+func (s *Schema) AddProcs(procs ...*Proc) *Schema {
+	for _, f := range procs {
+		f.Schema = s
+	}
+	s.Procs = append(s.Procs, procs...)
+	return s
+}
+
 // NewRealm creates a new Realm.
 func NewRealm(schemas ...*Schema) *Realm {
 	r := &Realm{Schemas: schemas}
@@ -231,6 +249,12 @@ func NewView(name, def string) *View {
 	return &View{Name: name, Def: def}
 }
 
+// NewMaterializedView creates a new materialized View.
+func NewMaterializedView(name, def string) *View {
+	return NewView(name, def).
+		SetMaterialized(true)
+}
+
 // SetSchema sets the schema (named-database) of the view.
 func (v *View) SetSchema(s *Schema) *View {
 	v.Schema = s
@@ -262,6 +286,15 @@ func (v *View) AddDeps(objs ...Object) *View {
 	return v
 }
 
+// AddIndexes appends the given indexes to the table index list.
+func (v *View) AddIndexes(indexes ...*Index) *View {
+	for _, idx := range indexes {
+		idx.View = v
+	}
+	v.Indexes = append(v.Indexes, indexes...)
+	return v
+}
+
 // SetCheckOption sets the check option of the view.
 func (v *View) SetCheckOption(opt string) *View {
 	ReplaceOrAppend(&v.Attrs, &ViewCheckOption{V: opt})
