@@ -102,6 +102,9 @@ type (
 	// ModifyView describes a view modification change.
 	ModifyView struct {
 		From, To *View
+		// Changes that are extra to the view definition.
+		// For example, adding or dropping indexes.
+		Changes []Change
 	}
 
 	// RenameView describes a view rename change.
@@ -109,6 +112,56 @@ type (
 		From, To *View
 	}
 
+	// AddFunc describes a function creation change.
+	AddFunc struct {
+		F     *Func
+		Extra []Clause // Extra clauses and options.
+	}
+
+	// DropFunc describes a function removal change.
+	DropFunc struct {
+		F     *Func
+		Extra []Clause // Extra clauses.
+	}
+
+	// ModifyFunc describes a function modification change.
+	ModifyFunc struct {
+		From, To *Func
+		// Changes that are extra to the function definition.
+		// For example, adding, dropping, or modifying attributes.
+		Changes []Change
+	}
+
+	// RenameFunc describes a function rename change.
+	RenameFunc struct {
+		From, To *Func
+	}
+
+	// AddProc describes a procedure creation change.
+	AddProc struct {
+		P     *Proc
+		Extra []Clause // Extra clauses and options.
+	}
+
+	// DropProc describes a procedure removal change.
+	DropProc struct {
+		P     *Proc
+		Extra []Clause // Extra clauses.
+	}
+
+	// ModifyProc describes a procedure modification change.
+	ModifyProc struct {
+		From, To *Proc
+		// Changes that are extra to the procedure definition.
+		// For example, adding, dropping, or modifying attributes.
+		Changes []Change
+	}
+
+	// RenameProc describes a procedure rename change.
+	RenameProc struct {
+		From, To *Proc
+	}
+
 	// AddObject describes a generic object creation change.
 	AddObject struct {
 		O     Object
@@ -541,6 +594,14 @@ func (*AddView) change()          {}
 func (*DropView) change()         {}
 func (*ModifyView) change()       {}
 func (*RenameView) change()       {}
+func (*AddFunc) change()          {}
+func (*DropFunc) change()         {}
+func (*ModifyFunc) change()       {}
+func (*RenameFunc) change()       {}
+func (*AddProc) change()          {}
+func (*DropProc) change()         {}
+func (*ModifyProc) change()       {}
+func (*RenameProc) change()       {}
 func (*AddObject) change()        {}
 func (*DropObject) change()       {}
 func (*ModifyObject) change()     {}
