@@ -18,6 +18,8 @@ type (
 		Realm   *Realm
 		Tables  []*Table
 		Views   []*View
+		Funcs   []*Func
+		Procs   []*Proc
 		Attrs   []Attr   // Attrs and options.
 		Objects []Object // Driver specific objects.
 	}
@@ -48,6 +50,7 @@ type (
 		Columns []*Column
 		Attrs   []Attr   // Attrs and options.
 		Deps    []Object // Tables and views used in view definition.
+		Indexes []*Index // Indexes on materialized view.
 	}
 
 	// A Column represents a column definition.
@@ -73,9 +76,11 @@ type (
 	Index struct {
 		Name   string
 		Unique bool
-		Table  *Table
-		Attrs  []Attr
-		Parts  []*IndexPart
+		// Table or View that this index belongs to.
+		Table *Table
+		View  *View
+		Attrs []Attr
+		Parts []*IndexPart
 	}
 
 	// An IndexPart represents an index part that
@@ -102,6 +107,47 @@ type (
 		OnUpdate   ReferenceOption
 		OnDelete   ReferenceOption
 	}
+
+	// Func represents a function definition.
+	Func struct {
+		Name   string
+		Schema *Schema
+		Args   []*FuncArg
+		Ret    Type
+		Body   string // Function body only.
+		Lang   string // Language (e.g. SQL, PL/pgSQL, etc.).
+		Attrs  []Attr // Extra driver specific attributes.
+	}
+
+	// Proc represents a procedure definition.
+	Proc struct {
+		Name   string
+		Schema *Schema
+		Args   []*FuncArg
+		Body   string // Function body only.
+		Lang   string // Language (e.g. SQL, PL/pgSQL, etc.).
+		Attrs  []Attr // Extra driver specific attributes.
+	}
+
+	// A FuncArg represents a single function argument.
+	FuncArg struct {
+		Name    string      // Optional name.
+		Type    Type        // Argument type.
+		Default Expr        // Default value.
+		Mode    FuncArgMode // Argument mode.
+		Attrs   []Attr      // Extra driver specific attributes.
+	}
+
+	// FuncArgMode represents a function argument mode.
+	FuncArgMode string
+)
+
+// List of supported function argument modes.
+const (
+	FuncArgModeIn       FuncArgMode = "IN"
+	FuncArgModeOut      FuncArgMode = "OUT"
+	FuncArgModeInOut    FuncArgMode = "INOUT"
+	FuncArgModeVariadic FuncArgMode = "VARIADIC"
 )
 
 // Schema returns the first schema that matched the given name.
@@ -126,9 +172,39 @@ func (s *Schema) Table(name string) (*Table, bool) {
 
 // View returns the first view that matched the given name.
 func (s *Schema) View(name string) (*View, bool) {
-	for _, t := range s.Views {
-		if t.Name == name {
-			return t, true
+	for _, v := range s.Views {
+		if v.Name == name && !v.Materialized() {
+			return v, true
+		}
+	}
+	return nil, false
+}
+
+// Materialized returns the first materialized view that matched the given name.
+func (s *Schema) Materialized(name string) (*View, bool) {
+	for _, v := range s.Views {
+		if v.Name == name && v.Materialized() {
+			return v, true
+		}
+	}
+	return nil, false
+}
+
+// Func returns the first function that matched the given name.
+func (s *Schema) Func(name string) (*Func, bool) {
+	for _, f := range s.Funcs {
+		if f.Name == name {
+			return f, true
+		}
+	}
+	return nil, false
+}
+
+// Proc returns the first procedure that matched the given name.
+func (s *Schema) Proc(name string) (*Proc, bool) {
+	for _, p := range s.Procs {
+		if p.Name == name {
+			return p, true
 		}
 	}
 	return nil, false
@@ -174,6 +250,26 @@ func (t *Table) ForeignKey(symbol string) (*ForeignKey, bool) {
 	return nil, false
 }
 
+// Materialized reports if the view is materialized.
+func (v *View) Materialized() bool {
+	for _, a := range v.Attrs {
+		if _, ok := a.(*Materialized); ok {
+			return true
+		}
+	}
+	return false
+}
+
+// SetMaterialized reports if the view is materialized.
+func (v *View) SetMaterialized(b bool) *View {
+	if b {
+		ReplaceOrAppend(&v.Attrs, &Materialized{})
+	} else {
+		v.Attrs = RemoveAttr[*Materialized](v.Attrs)
+	}
+	return v
+}
+
 // Column returns the first column that matched the given name.
 func (v *View) Column(name string) (*Column, bool) {
 	for _, c := range v.Columns {
@@ -184,6 +280,23 @@ func (v *View) Column(name string) (*Column, bool) {
 	return nil, false
 }
 
+// Index returns the first index that matched the given name.
+func (v *View) Index(name string) (*Index, bool) {
+	for _, i := range v.Indexes {
+		if i.Name == name {
+			return i, true
+		}
+	}
+	return nil, false
+}
+
+// AsTable returns a table that represents the view.
+func (v *View) AsTable() *Table {
+	return NewTable(v.Name).
+		SetSchema(v.Schema).
+		AddColumns(v.Columns...)
+}
+
 // Column returns the first column that matches the given name.
 func (f *ForeignKey) Column(name string) (*Column, bool) {
 	for _, c := range f.Columns {
@@ -285,6 +398,7 @@ type (
 		T         string
 		Precision *int
 		Scale     *int
+		Attrs     []Attr
 	}
 
 	// JSONType represents a JSON type.
@@ -377,6 +491,12 @@ type (
 	ViewCheckOption struct {
 		V string // LOCAL, CASCADED, NONE, or driver specific.
 	}
+
+	// Materialized is a schema attribute that attached to views to indicates
+	// they are MATERIALIZED VIEWs.
+	Materialized struct {
+		Attr
+	}
 )
 
 // A list of known view check options.
