@@ -7,11 +7,13 @@ package sqltool
 import (
 	"bufio"
 	"bytes"
+	"errors"
 	"fmt"
 	"io/fs"
 	"path/filepath"
 	"regexp"
 	"sort"
+	"strconv"
 	"strings"
 	"text/template"
 	"time"
@@ -70,9 +72,9 @@ var (
 )
 
 type (
-	// GolangMigrateDir wraps migrate.LocalDir and provides a migrate.Scanner implementation able to understand files
+	// GolangMigrateDir wraps fs.FS and provides a migrate.Scanner implementation able to understand files
 	// generated by the GolangMigrateFormatter for migration directory replaying.
-	GolangMigrateDir struct{ *migrate.LocalDir }
+	GolangMigrateDir struct{ fs.FS }
 	// GolangMigrateFile wraps migrate.LocalFile with custom description function.
 	GolangMigrateFile struct{ *migrate.LocalFile }
 )
@@ -86,6 +88,14 @@ func NewGolangMigrateDir(path string) (*GolangMigrateDir, error) {
 	return &GolangMigrateDir{dir}, nil
 }
 
+// Path returns the local path used for opening this dir.
+func (d *GolangMigrateDir) Path() string {
+	if dir, ok := d.FS.(dirPath); ok {
+		return dir.Path()
+	}
+	return ""
+}
+
 // Files implements Scanner.Files. It looks for all files with up.sql suffix and orders them by filename.
 func (d *GolangMigrateDir) Files() ([]migrate.File, error) {
 	names, err := fs.Glob(d, "*.up.sql")
@@ -107,6 +117,26 @@ func (d *GolangMigrateDir) Files() ([]migrate.File, error) {
 	return ret, nil
 }
 
+// Checksum implements Dir.Checksum. By default, it calls Files() and creates a checksum from them.
+func (d *GolangMigrateDir) Checksum() (migrate.HashFile, error) {
+	if d, ok := d.FS.(migrate.Dir); ok {
+		return d.Checksum()
+	}
+	files, err := d.Files()
+	if err != nil {
+		return nil, err
+	}
+	return migrate.NewHashFile(files)
+}
+
+// WriteFile implements Dir.WriteFile.
+func (d *GolangMigrateDir) WriteFile(name string, b []byte) error {
+	if d, ok := d.FS.(migrate.Dir); ok {
+		return d.WriteFile(name, b)
+	}
+	return errors.New("sql/sqltool: write not supported")
+}
+
 // Desc implements File.Desc.
 func (f *GolangMigrateFile) Desc() string {
 	return strings.TrimSuffix(f.LocalFile.Desc(), ".up")
@@ -290,9 +320,9 @@ func (f *DBMateFile) Stmts() ([]string, error) {
 }
 
 type (
-	// FlywayDir wraps migrate.LocalDir and provides a migrate.Scanner implementation able to understand files
+	// FlywayDir wraps fs.FS and provides a migrate.Scanner implementation able to understand files
 	// generated by the FlywayFormatter for migration directory replaying.
-	FlywayDir struct{ *migrate.LocalDir }
+	FlywayDir struct{ fs.FS }
 	// FlywayFile wraps migrate.LocalFile with custom statements function.
 	FlywayFile struct{ *migrate.LocalFile }
 )
@@ -306,17 +336,29 @@ func NewFlywayDir(path string) (*FlywayDir, error) {
 	return &FlywayDir{dir}, nil
 }
 
+// Path returns the local path used for opening this dir.
+func (d *FlywayDir) Path() string {
+	if dir, ok := d.FS.(dirPath); ok {
+		return dir.Path()
+	}
+	return ""
+}
+
 // Files implements Scanner.Files. It looks for all files with .sql suffix. The given directory is recursively scanned
 // for non-hidden subdirectories. All found files will be ordered by migration type (Baseline, Versioned, Repeatable)
 // and filename.
 func (d *FlywayDir) Files() ([]migrate.File, error) {
 	var ff flywayFiles
-	if err := fs.WalkDir(d, "", func(path string, e fs.DirEntry, err error) error {
+	if err := fs.WalkDir(d, ".", func(path string, e fs.DirEntry, err error) error {
 		if err != nil {
 			return err
 		}
-		if path != "" && e.IsDir() {
-			h, err := hidden(filepath.Join(d.Path(), path))
+		if path != "." && e.IsDir() {
+			fullPath := path
+			if p, ok := d.FS.(dirPath); ok {
+				fullPath = filepath.Join(p.Path(), path)
+			}
+			h, err := hidden(fullPath)
 			if err != nil {
 				return err
 			}
@@ -351,6 +393,26 @@ func (d *FlywayDir) Files() ([]migrate.File, error) {
 	return ret, nil
 }
 
+// Checksum implements Dir.Checksum. By default, it calls Files() and creates a checksum from them.
+func (d *FlywayDir) Checksum() (migrate.HashFile, error) {
+	if d, ok := d.FS.(migrate.Dir); ok {
+		return d.Checksum()
+	}
+	files, err := d.Files()
+	if err != nil {
+		return nil, err
+	}
+	return migrate.NewHashFile(files)
+}
+
+// WriteFile implements Dir.WriteFile.
+func (d *FlywayDir) WriteFile(name string, b []byte) error {
+	if d, ok := d.FS.(migrate.Dir); ok {
+		return d.WriteFile(name, b)
+	}
+	return errors.New("sql/sqltool: write not supported")
+}
+
 // Desc implements File.Desc.
 func (f FlywayFile) Desc() string {
 	return flywayDesc(f.Name())
@@ -473,8 +535,8 @@ func (ff *flywayFiles) names() []string {
 	if ff.baseline != "" {
 		names = append(names, ff.baseline)
 	}
-	sort.Strings(ff.versioned)
-	sort.Strings(ff.repeatable)
+	flywaySort(ff.versioned)
+	flywaySort(ff.repeatable)
 	names = append(names, ff.versioned...)
 	names = append(names, ff.repeatable...)
 	return names
@@ -496,6 +558,26 @@ func flywayVersion(path string) string {
 	return strings.SplitN(strings.TrimSuffix(filepath.Base(path), ".sql"), "__", 2)[0][1:]
 }
 
+func flywaySort(files []string) {
+	sort.Slice(files, func(i, j int) bool {
+		return flywayVersionCompare(flywayVersion(files[i]), flywayVersion(files[j])) < 0
+	})
+}
+
+func flywayVersionCompare(v1, v2 string) int {
+	parse := func(s string) []int {
+		ss := strings.Split(strings.ReplaceAll(s, "_", "."), ".")
+		var ret []int
+		for _, s := range ss {
+			// 0 for non-numeric parts.
+			i, _ := strconv.Atoi(s)
+			ret = append(ret, i)
+		}
+		return ret
+	}
+	return compareSliceInt(parse(v1), parse(v2))
+}
+
 func unexpectedPragmaErr(f migrate.File, line int, pragma string) error {
 	var tool string
 	switch f := f.(type) {
@@ -545,3 +627,36 @@ func reverse(changes []*migrate.Change) []*migrate.Change {
 	}
 	return rev
 }
+
+type dirPath interface {
+	Path() string
+}
+
+var (
+	_ dirPath = (*DBMateDir)(nil)
+	_ dirPath = (*FlywayDir)(nil)
+	_ dirPath = (*GolangMigrateDir)(nil)
+	_ dirPath = (*GooseDir)(nil)
+	_ dirPath = (*LiquibaseDir)(nil)
+)
+
+// Copied from golang.org/x/exp/slices.Compare
+func compareSliceInt(s1, s2 []int) int {
+	s2len := len(s2)
+	for i, v1 := range s1 {
+		if i >= s2len {
+			return +1
+		}
+		v2 := s2[i]
+		switch {
+		case v1 < v2:
+			return -1
+		case v1 > v2:
+			return +1
+		}
+	}
+	if len(s1) < s2len {
+		return -1
+	}
+	return 0
+}
